@@ -31,7 +31,9 @@ import java.util.Locale;
 
 import org.apache.http.HttpStatus;
 import org.apache.http.ReasonPhraseCatalog;
-import org.apache.http.annotation.Immutable;
+import org.apache.http.annotation.ThreadingBehavior;
+import org.apache.http.annotation.Contract;
+import org.apache.http.util.Args;
 
 /**
  * English reason phrases for HTTP status codes.
@@ -40,7 +42,7 @@ import org.apache.http.annotation.Immutable;
  *
  * @since 4.0
  */
-@Immutable
+@Contract(threading = ThreadingBehavior.IMMUTABLE)
 public class EnglishReasonPhraseCatalog implements ReasonPhraseCatalog {
 
     // static array with english reason phrases defined below
@@ -50,8 +52,7 @@ public class EnglishReasonPhraseCatalog implements ReasonPhraseCatalog {
      * This catalog is thread safe, so there typically
      * is no need to create other instances.
      */
-    public final static EnglishReasonPhraseCatalog INSTANCE =
-        new EnglishReasonPhraseCatalog();
+    public final static EnglishReasonPhraseCatalog INSTANCE = new EnglishReasonPhraseCatalog();
 
 
     /**
@@ -69,20 +70,18 @@ public class EnglishReasonPhraseCatalog implements ReasonPhraseCatalog {
      * @param status    the status code, in the range 100-599
      * @param loc       ignored
      *
-     * @return  the reason phrase, or <code>null</code>
+     * @return  the reason phrase, or {@code null}
      */
-    public String getReason(int status, Locale loc) {
-        if ((status < 100) || (status >= 600)) {
-            throw new IllegalArgumentException
-                ("Unknown category for status code " + status + ".");
-        }
-
+    @Override
+    public String getReason(final int status, final Locale loc) {
+        Args.check(status >= 100 && status < 600, "Unknown category for status code " + status);
         final int category = status / 100;
         final int subcode  = status - 100*category;
 
         String reason = null;
-        if (REASON_PHRASES[category].length > subcode)
+        if (REASON_PHRASES[category].length > subcode) {
             reason = REASON_PHRASES[category][subcode];
+        }
 
         return reason;
     }
@@ -94,7 +93,7 @@ public class EnglishReasonPhraseCatalog implements ReasonPhraseCatalog {
         new String[3],  // 1xx
         new String[8],  // 2xx
         new String[8],  // 3xx
-        new String[25], // 4xx
+        new String[30], // 4xx
         new String[8]   // 5xx
     };
 
@@ -107,7 +106,7 @@ public class EnglishReasonPhraseCatalog implements ReasonPhraseCatalog {
      * @param status    the status code for which to define the phrase
      * @param reason    the reason phrase for this status code
      */
-    private static void setReason(int status, String reason) {
+    private static void setReason(final int status, final String reason) {
         final int category = status / 100;
         final int subcode  = status - 100*category;
         REASON_PHRASES[category][subcode] = reason;
@@ -141,6 +140,8 @@ public class EnglishReasonPhraseCatalog implements ReasonPhraseCatalog {
                   "Forbidden");
         setReason(HttpStatus.SC_NOT_FOUND,
                   "Not Found");
+        setReason(HttpStatus.SC_TOO_MANY_REQUESTS,
+                  "Too Many Requests");
         setReason(HttpStatus.SC_INTERNAL_SERVER_ERROR,
                   "Internal Server Error");
         setReason(HttpStatus.SC_NOT_IMPLEMENTED,
