@@ -6,7 +6,7 @@
  *  (the "License"); you may not use this file except in compliance with
  *  the License.  You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -46,8 +46,8 @@ import org.apache.tools.ant.ExitException;
  */
 public class Permissions {
 
-    private final List<Permission> grantedPermissions = new LinkedList<Permission>();
-    private final List<Permission> revokedPermissions = new LinkedList<Permission>();
+    private final List<Permission> grantedPermissions = new LinkedList<>();
+    private final List<Permission> revokedPermissions = new LinkedList<>();
     private java.security.Permissions granted = null;
     private SecurityManager origSm = null;
     private boolean active = false;
@@ -157,7 +157,7 @@ public class Permissions {
             final String name = permission.getName();
             final String actions = permission.getActions();
             final Constructor<? extends java.security.Permission> ctr = clazz.getConstructor(PARAMS);
-            return ctr.newInstance(new Object[] {name, actions});
+            return ctr.newInstance(name, actions);
         } catch (final Exception e) {
             // Let the UnresolvedPermission handle it.
             return new UnresolvedPermission(permission.getClassName(),
@@ -207,10 +207,7 @@ public class Permissions {
         public void checkPermission(final java.security.Permission perm) {
             if (active) {
                 if (delegateToOldSM && !perm.getName().equals("exitVM")) {
-                    boolean permOK = false;
-                    if (granted.implies(perm)) {
-                        permOK = true;
-                    }
+                    boolean permOK = granted.implies(perm);
                     checkRevoked(perm);
                     /*
                      if the permission was not explicitly granted or revoked
@@ -286,7 +283,7 @@ public class Permissions {
          */
         public void setActions(final String actions) {
             actionString = actions;
-            if (actions.length() > 0) {
+            if (!actions.isEmpty()) {
                 this.actions = parseActions(actions);
             }
         }
@@ -312,20 +309,16 @@ public class Permissions {
                     if (!perm.getName().startsWith(name.substring(0, name.length() - 1))) {
                         return false;
                     }
-                } else {
-                    if (!name.equals(perm.getName())) {
-                        return false;
-                    }
+                } else if (!name.equals(perm.getName())) {
+                    return false;
                 }
             }
             if (actions != null) {
                 final Set<String> as = parseActions(perm.getActions());
                 final int size = as.size();
                 as.removeAll(actions);
-                if (as.size() == size) {
-                    // None of the actions revoked, so all allowed.
-                    return false;
-                }
+                // If no actions removed, then all allowed
+                return as.size() != size;
             }
             return true;
         }
@@ -335,11 +328,11 @@ public class Permissions {
          * @param actions The actions to be parsed.
          */
         private Set<String> parseActions(final String actions) {
-            final Set<String> result = new HashSet<String>();
+            final Set<String> result = new HashSet<>();
             final StringTokenizer tk = new StringTokenizer(actions, ",");
             while (tk.hasMoreTokens()) {
                 final String item = tk.nextToken().trim();
-                if (!item.equals("")) {
+                if (!item.isEmpty()) {
                     result.add(item);
                 }
             }
