@@ -16,11 +16,12 @@
 
 package javassist.bytecode;
 
+import java.util.Map;
+
 import javassist.ClassPool;
 import javassist.CtClass;
 import javassist.CtPrimitiveType;
 import javassist.NotFoundException;
-import java.util.Map;
 
 /**
  * A support class for dealing with descriptors.
@@ -59,8 +60,7 @@ public class Descriptor {
     public static String toJvmName(CtClass clazz) {
         if (clazz.isArray())
             return of(clazz);
-        else
-            return toJvmName(clazz.getName());
+        return toJvmName(clazz.getName());
     }
 
     /**
@@ -109,14 +109,12 @@ public class Descriptor {
 
         if (arrayDim == 0)
             return name;
-        else {
-            StringBuffer sbuf = new StringBuffer(name);
-            do {
-                sbuf.append("[]");
-            } while (--arrayDim > 0);
+        StringBuilder sbuf = new StringBuilder(name);
+        do {
+            sbuf.append("[]");
+        } while (--arrayDim > 0);
 
-            return sbuf.toString();
-        }
+        return sbuf.toString();
     }
 
     /**
@@ -159,7 +157,7 @@ public class Descriptor {
         if (desc.indexOf(oldname) < 0)
             return desc;
 
-        StringBuffer newdesc = new StringBuffer();
+        StringBuilder newdesc = new StringBuilder();
         int head = 0;
         int i = 0;
         for (;;) {
@@ -183,13 +181,11 @@ public class Descriptor {
 
         if (head == 0)
             return desc;
-        else {
-            int len = desc.length();
-            if (head < len)
-                newdesc.append(desc.substring(head, len));
+        int len = desc.length();
+        if (head < len)
+            newdesc.append(desc.substring(head, len));
 
-            return newdesc.toString();
-        }
+        return newdesc.toString();
     }
 
     /**
@@ -200,11 +196,11 @@ public class Descriptor {
      *            JVM class names.
      * @see Descriptor#toJvmName(String)
      */
-    public static String rename(String desc, Map map) {
+    public static String rename(String desc, Map<String,String> map) {
         if (map == null)
             return desc;
 
-        StringBuffer newdesc = new StringBuffer();
+        StringBuilder newdesc = new StringBuilder();
         int head = 0;
         int i = 0;
         for (;;) {
@@ -218,7 +214,7 @@ public class Descriptor {
 
             i = k + 1;
             String name = desc.substring(j + 1, k);
-            String name2 = (String)map.get(name);
+            String name2 = map.get(name);
             if (name2 != null) {
                 newdesc.append(desc.substring(head, j));
                 newdesc.append('L');
@@ -230,25 +226,23 @@ public class Descriptor {
 
         if (head == 0)
             return desc;
-        else {
-            int len = desc.length();
-            if (head < len)
-                newdesc.append(desc.substring(head, len));
+        int len = desc.length();
+        if (head < len)
+            newdesc.append(desc.substring(head, len));
 
-            return newdesc.toString();
-        }
+        return newdesc.toString();
     }
 
     /**
      * Returns the descriptor representing the given type.
      */
     public static String of(CtClass type) {
-        StringBuffer sbuf = new StringBuffer();
+        StringBuilder sbuf = new StringBuilder();
         toDescriptor(sbuf, type);
         return sbuf.toString();
     }
 
-    private static void toDescriptor(StringBuffer desc, CtClass type) {
+    private static void toDescriptor(StringBuilder desc, CtClass type) {
         if (type.isArray()) {
             desc.append('[');
             try {
@@ -290,7 +284,7 @@ public class Descriptor {
      * @param paramTypes parameter types
      */
     public static String ofMethod(CtClass returnType, CtClass[] paramTypes) {
-        StringBuffer desc = new StringBuffer();
+        StringBuilder desc = new StringBuilder();
         desc.append('(');
         if (paramTypes != null) {
             int n = paramTypes.length;
@@ -329,15 +323,13 @@ public class Descriptor {
         int i = desc.indexOf(')');
         if (i < 0)
             return desc;
-        else {
-            StringBuffer newdesc = new StringBuffer();
-            newdesc.append(desc.substring(0, i));
-            newdesc.append('L');
-            newdesc.append(classname.replace('.', '/'));
-            newdesc.append(';');
-            newdesc.append(desc.substring(i));
-            return newdesc.toString();
-        }
+        StringBuilder newdesc = new StringBuilder();
+        newdesc.append(desc.substring(0, i));
+        newdesc.append('L');
+        newdesc.append(classname.replace('.', '/'));
+        newdesc.append(';');
+        newdesc.append(desc.substring(i));
+        return newdesc.toString();
     }
 
     /**
@@ -353,9 +345,8 @@ public class Descriptor {
     public static String insertParameter(String classname, String desc) {
         if (desc.charAt(0) != '(')
             return desc;
-        else
-            return "(L" + classname.replace('.', '/') + ';'
-                   + desc.substring(1);
+        return "(L" + classname.replace('.', '/') + ';'
+               + desc.substring(1);
     }
 
     /**
@@ -370,13 +361,11 @@ public class Descriptor {
         int i = descriptor.indexOf(')');
         if (i < 0)
             return descriptor;
-        else {
-            StringBuffer newdesc = new StringBuffer();
-            newdesc.append(descriptor.substring(0, i));
-            toDescriptor(newdesc, type);
-            newdesc.append(descriptor.substring(i));
-            return newdesc.toString();
-        }
+        StringBuilder newdesc = new StringBuilder();
+        newdesc.append(descriptor.substring(0, i));
+        toDescriptor(newdesc, type);
+        newdesc.append(descriptor.substring(i));
+        return newdesc.toString();
     }
 
     /**
@@ -391,8 +380,7 @@ public class Descriptor {
                                          String descriptor) {
         if (descriptor.charAt(0) != '(')
             return descriptor;
-        else
-            return "(" + of(type) + descriptor.substring(1);
+        return "(" + of(type) + descriptor.substring(1);
     }
 
     /**
@@ -407,14 +395,12 @@ public class Descriptor {
         int i = desc.indexOf(')');
         if (i < 0)
             return desc;
-        else {
-            StringBuffer newdesc = new StringBuffer();
-            newdesc.append(desc.substring(0, i + 1));
-            newdesc.append('L');
-            newdesc.append(classname.replace('.', '/'));
-            newdesc.append(';');
-            return newdesc.toString();
-        }
+        StringBuilder newdesc = new StringBuilder();
+        newdesc.append(desc.substring(0, i + 1));
+        newdesc.append('L');
+        newdesc.append(classname.replace('.', '/'));
+        newdesc.append(';');
+        return newdesc.toString();
     }
 
     /**
@@ -430,16 +416,14 @@ public class Descriptor {
     {
         if (desc.charAt(0) != '(')
             return null;
-        else {
-            int num = numOfParameters(desc);
-            CtClass[] args = new CtClass[num];
-            int n = 0;
-            int i = 1;
-            do {
-                i = toCtClass(cp, desc, i, args, n++);
-            } while (i > 0);
-            return args;
-        }
+        int num = numOfParameters(desc);
+        CtClass[] args = new CtClass[num];
+        int n = 0;
+        int i = 1;
+        do {
+            i = toCtClass(cp, desc, i, args, n++);
+        } while (i > 0);
+        return args;
     }
 
     /**
@@ -484,11 +468,9 @@ public class Descriptor {
         int i = desc.indexOf(')');
         if (i < 0)
             return null;
-        else {
-            CtClass[] type = new CtClass[1];
-            toCtClass(cp, desc, i + 1, type, 0);
-            return type[0];
-        }
+        CtClass[] type = new CtClass[1];
+        toCtClass(cp, desc, i + 1, type, 0);
+        return type[0];
     }
 
     /**
@@ -542,11 +524,9 @@ public class Descriptor {
         int res = toCtClass(cp, desc, 0, clazz, 0);
         if (res >= 0)
             return clazz[0];
-        else {
-            // maybe, you forgot to surround the class name with
-            // L and ;.  It violates the protocol, but I'm tolerant...
-            return cp.get(desc.replace('/', '.'));
-        }
+        // maybe, you forgot to surround the class name with
+        // L and ;.  It violates the protocol, but I'm tolerant...
+        return cp.get(desc.replace('/', '.'));
     }
 
     private static int toCtClass(ClassPool cp, String desc, int i,
@@ -577,12 +557,11 @@ public class Descriptor {
                 args[n] = type;
                 return i2; // neither an array type or a class type
             }
-            else
-                name = type.getName();
+            name = type.getName();
         }
 
         if (arrayDim > 0) {
-            StringBuffer sbuf = new StringBuffer(name);
+            StringBuilder sbuf = new StringBuilder(name);
             while (arrayDim-- > 0)
                 sbuf.append("[]");
 
@@ -740,7 +719,7 @@ public class Descriptor {
 
     static class PrettyPrinter {
         static String toString(String desc) {
-            StringBuffer sbuf = new StringBuffer();
+            StringBuilder sbuf = new StringBuilder();
             if (desc.charAt(0) == '(') {
                 int pos = 1;
                 sbuf.append('(');
@@ -759,7 +738,7 @@ public class Descriptor {
             return sbuf.toString();
         }
 
-        static int readType(StringBuffer sbuf, int pos, String desc) {
+        static int readType(StringBuilder sbuf, int pos, String desc) {
             char c = desc.charAt(pos);
             int arrayDim = 0;
             while (c == '[') {
