@@ -89,6 +89,8 @@ public class CompilerConfiguration {
     public static final String JDK15 = "15";
     /** This (<code>"16"</code>) is the value for targetBytecode to compile for a JDK 16. */
     public static final String JDK16 = "16";
+    /** This (<code>"17"</code>) is the value for targetBytecode to compile for a JDK 17. */
+    public static final String JDK17 = "17";
 
     /**
      * This constant is for comparing targetBytecode to ensure it is set to JDK 1.5 or later.
@@ -120,7 +122,8 @@ public class CompilerConfiguration {
             JDK13, Opcodes.V13,
             JDK14, Opcodes.V14,
             JDK15, Opcodes.V15,
-            JDK16, Opcodes.V16
+            JDK16, Opcodes.V16,
+            JDK17, Opcodes.V17
     );
 
     /**
@@ -129,13 +132,10 @@ public class CompilerConfiguration {
     public static final String[] ALLOWED_JDKS = JDK_TO_BYTECODE_VERSION_MAP.keySet().toArray(new String[JDK_TO_BYTECODE_VERSION_MAP.size()]);
 
     /**
-    * The ASM api version to use when loading/parsing classes, and generating proxy adapter classes.
-    */
-    public static final int ASM_API_VERSION = Opcodes.ASM8;
-
-    /**
-     * The default source encoding.
+     * The ASM API version used when loading/parsing classes and generating proxy adapter classes.
      */
+    public static final int ASM_API_VERSION = Opcodes.ASM9;
+
     public static final String DEFAULT_SOURCE_ENCODING = "UTF-8";
 
     /**
@@ -443,21 +443,10 @@ public class CompilerConfiguration {
         defaultScriptExtension = getSystemPropertySafe("groovy.default.scriptExtension", ".groovy");
 
         optimizationOptions = new HashMap<>(4);
-        handleOptimizationOption(optimizationOptions, INVOKEDYNAMIC, "groovy.target.indy");
-        handleOptimizationOption(optimizationOptions, GROOVYDOC, "groovy.attach.groovydoc");
-        handleOptimizationOption(optimizationOptions, RUNTIME_GROOVYDOC, "groovy.attach.runtime.groovydoc");
-        handleOptimizationOption(optimizationOptions, PARALLEL_PARSE, "groovy.parallel.parse");
-    }
-
-    private void handleOptimizationOption(final Map<String, Boolean> options, final String optionName, final String sysOptionName) {
-        String propValue = getSystemPropertySafe(sysOptionName);
-        boolean optionEnabled = propValue == null
-                ? (DEFAULT != null && Boolean.TRUE.equals(DEFAULT.getOptimizationOptions().get(optionName)))
-                : Boolean.parseBoolean(propValue);
-
-        if (optionEnabled) {
-            options.put(optionName, Boolean.TRUE);
-        }
+        if (getBooleanSafe("groovy.target.indy"             )) optimizationOptions.put(INVOKEDYNAMIC    , Boolean.TRUE);
+        if (getBooleanSafe("groovy.attach.groovydoc"        )) optimizationOptions.put(GROOVYDOC        , Boolean.TRUE);
+        if (getBooleanSafe("groovy.attach.runtime.groovydoc")) optimizationOptions.put(RUNTIME_GROOVYDOC, Boolean.TRUE);
+        if (getBooleanSafe("groovy.parallel.parse"          )) optimizationOptions.put(PARALLEL_PARSE   , Boolean.TRUE);
     }
 
     /**
@@ -479,7 +468,7 @@ public class CompilerConfiguration {
     public CompilerConfiguration(final CompilerConfiguration configuration) {
         setWarningLevel(configuration.getWarningLevel());
         setTargetDirectory(configuration.getTargetDirectory());
-        setClasspathList(new LinkedList<String>(configuration.getClasspath()));
+        setClasspathList(configuration.getClasspath());
         setVerbose(configuration.getVerbose());
         setDebug(configuration.getDebug());
         setParameters(configuration.getParameters());
@@ -743,11 +732,7 @@ public class CompilerConfiguration {
      */
     @Deprecated
     public void setOutput(final PrintWriter output) {
-        if (output == null) {
-            this.output = new PrintWriter(NullWriter.DEFAULT);
-        } else {
-            this.output = output;
-        }
+        this.output = (output != null ? output : new PrintWriter(NullWriter.DEFAULT));
     }
 
     /**
@@ -757,6 +742,13 @@ public class CompilerConfiguration {
         return this.targetDirectory;
     }
 
+    /**
+     * Sets the target directory.
+     */
+    public void setTargetDirectory(final File directory) {
+        this.targetDirectory = directory;
+    }
+
     /**
      * Sets the target directory.
      */
@@ -772,13 +764,6 @@ public class CompilerConfiguration {
         }
     }
 
-    /**
-     * Sets the target directory.
-     */
-    public void setTargetDirectory(final File directory) {
-        this.targetDirectory = directory;
-    }
-
     /**
      * @return the classpath
      */
@@ -933,8 +918,18 @@ public class CompilerConfiguration {
     }
 
     /**
-     * Sets the bytecode compatibility level. The parameter can take one of the values
-     * in {@link #ALLOWED_JDKS}.
+     * Returns the compiler bytecode compatibility level. Defaults to the minimum
+     * officially supported bytecode version for any particular Groovy version.
+     *
+     * @return bytecode compatibility level
+     */
+    public String getTargetBytecode() {
+        return targetBytecode;
+    }
+
+    /**
+     * Sets the bytecode compatibility level. The parameter can take one of the
+     * values in {@link #ALLOWED_JDKS}.
      *
      * @param version the bytecode compatibility level
      */
@@ -948,16 +943,6 @@ public class CompilerConfiguration {
         }
     }
 
-    /**
-     * Retrieves the compiler bytecode compatibility level. Defaults to the minimum
-     * officially supported bytecode version for any particular Groovy version.
-     *
-     * @return bytecode compatibility level
-     */
-    public String getTargetBytecode() {
-        return this.targetBytecode;
-    }
-
     /**
      * Whether the bytecode version has preview features enabled (JEP 12)
      *
@@ -1072,23 +1057,20 @@ public class CompilerConfiguration {
      * Checks if invoke dynamic is enabled.
      */
     public boolean isIndyEnabled() {
-        Boolean indyEnabled = getOptimizationOptions().get(INVOKEDYNAMIC);
-        return Optional.ofNullable(indyEnabled).orElse(Boolean.FALSE);
+        return Boolean.TRUE.equals(getOptimizationOptions().get(INVOKEDYNAMIC));
     }
 
     /**
      * Checks if groovydoc is enabled.
      */
     public boolean isGroovydocEnabled() {
-        Boolean groovydocEnabled = getOptimizationOptions().get(GROOVYDOC);
-        return Optional.ofNullable(groovydocEnabled).orElse(Boolean.FALSE);
+        return Boolean.TRUE.equals(getOptimizationOptions().get(GROOVYDOC));
     }
 
     /**
      * Checks if runtime groovydoc is enabled.
      */
     public boolean isRuntimeGroovydocEnabled() {
-        Boolean runtimeGroovydocEnabled = getOptimizationOptions().get(RUNTIME_GROOVYDOC);
-        return Optional.ofNullable(runtimeGroovydocEnabled).orElse(Boolean.FALSE);
+        return Boolean.TRUE.equals(getOptimizationOptions().get(RUNTIME_GROOVYDOC));
     }
 }
