@@ -43,6 +43,7 @@ import org.codehaus.groovy.vmplugin.VMPluginFactory;
 
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Field;
+import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Modifier;
 import java.util.ArrayList;
 import java.util.HashMap;
@@ -53,13 +54,13 @@ import java.util.Map;
 import java.util.Properties;
 
 /**
- * A registry of MetaClass instances which caches introspection &
+ * A registry of MetaClass instances which caches introspection and
  * reflection information and allows methods to be dynamically added to
  * existing classes at runtime
  */
 public class MetaClassRegistryImpl implements MetaClassRegistry{
     /**
-     * @deprecated Use {@link ExtensionModuleScanner#MODULE_META_INF_FILE instead}
+     * @deprecated Use {@link ExtensionModuleScanner#MODULE_META_INF_FILE} instead
      */
     @Deprecated
     public static final String MODULE_META_INF_FILE = "META-INF/services/org.codehaus.groovy.runtime.ExtensionModule";
@@ -96,7 +97,7 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
     public MetaClassRegistryImpl(boolean useAccessible) {
         this(LOAD_DEFAULT, useAccessible);
     }
-    
+
     public MetaClassRegistryImpl(final int loadDefault, final boolean useAccessible) {
         this.useAccessible = useAccessible;
 
@@ -134,25 +135,23 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
         ClassInfo.getClassInfo(ExpandoMetaClass.class).setStrongMetaClass(emcMetaClass);
 
 
-        addNonRemovableMetaClassRegistryChangeEventListener(new MetaClassRegistryChangeEventListener(){
-            public void updateConstantMetaClass(MetaClassRegistryChangeEvent cmcu) {
-                // The calls to DefaultMetaClassInfo.setPrimitiveMeta and sdyn.setBoolean need to be
-                // ordered. Even though metaClassInfo is thread-safe, it is included in the block
-                // so the meta classes are added to the queue in the same order.
-                synchronized (metaClassInfo) {
-                   metaClassInfo.add(cmcu.getNewMetaClass());
-                   DefaultMetaClassInfo.getNewConstantMetaClassVersioning();
-                   Class c = cmcu.getClassToUpdate();
-                   DefaultMetaClassInfo.setPrimitiveMeta(c, cmcu.getNewMetaClass()==null);
-                   Field sdyn;
-                   try {
-                       sdyn = c.getDeclaredField(Verifier.STATIC_METACLASS_BOOL);
-                       sdyn.setBoolean(null, cmcu.getNewMetaClass()!=null);
-                   } catch (Throwable e) {
-                       //DO NOTHING
-                   }
+        addNonRemovableMetaClassRegistryChangeEventListener(cmcu -> {
+            // The calls to DefaultMetaClassInfo.setPrimitiveMeta and sdyn.setBoolean need to be
+            // ordered. Even though metaClassInfo is thread-safe, it is included in the block
+            // so the meta classes are added to the queue in the same order.
+            synchronized (metaClassInfo) {
+               metaClassInfo.add(cmcu.getNewMetaClass());
+               DefaultMetaClassInfo.getNewConstantMetaClassVersioning();
+               Class c = cmcu.getClassToUpdate();
+               DefaultMetaClassInfo.setPrimitiveMeta(c, cmcu.getNewMetaClass()==null);
+               Field sdyn;
+               try {
+                   sdyn = c.getDeclaredField(Verifier.STATIC_METACLASS_BOOL);
+                   sdyn.setBoolean(null, cmcu.getNewMetaClass()!=null);
+               } catch (Throwable e) {
+                   //DO NOTHING
+               }
 
-                }
             }
         });
    }
@@ -190,7 +189,7 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
                throw new GroovyRuntimeException("Could not instantiate custom Metaclass creation handle: "+ e, e);
            }
     }
-    
+
     private void registerMethods(final Class theClass, final boolean useMethodWrapper, final boolean useInstanceMethods, Map<CachedClass, List<MetaMethod>> map) {
         if (useMethodWrapper) {
             // Here we instantiate objects representing MetaMethods for DGM methods.
@@ -201,7 +200,7 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
 
                 for (GeneratedMetaMethod.DgmMethodRecord record : records) {
                     Class[] newParams = new Class[record.parameters.length - 1];
-                    System.arraycopy(record.parameters, 1, newParams, 0, record.parameters.length-1);
+                    System.arraycopy(record.parameters, 1, newParams, 0, newParams.length);
 
                     MetaMethod method = new GeneratedMetaMethod.Proxy(
                             record.className,
@@ -211,11 +210,7 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
                             newParams
                     );
                     final CachedClass declClass = method.getDeclaringClass();
-                    List<MetaMethod> arr = map.get(declClass);
-                    if (arr == null) {
-                        arr = new ArrayList<MetaMethod>(4);
-                        map.put(declClass, arr);
-                    }
+                    List<MetaMethod> arr = map.computeIfAbsent(declClass, k -> new ArrayList<MetaMethod>(4));
                     arr.add(method);
                     instanceMethods.add(method);
                 }
@@ -229,14 +224,10 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
 
             for (CachedMethod method : methods) {
                 final int mod = method.getModifiers();
-                if (Modifier.isStatic(mod) && Modifier.isPublic(mod) && method.getCachedMethod().getAnnotation(Deprecated.class) == null) {
+                if (Modifier.isStatic(mod) && Modifier.isPublic(mod) && method.getAnnotation(Deprecated.class) == null) {
                     CachedClass[] paramTypes = method.getParameterTypes();
                     if (paramTypes.length > 0) {
-                        List<MetaMethod> arr = map.get(paramTypes[0]);
-                        if (arr == null) {
-                            arr = new ArrayList<MetaMethod>(4);
-                            map.put(paramTypes[0], arr);
-                        }
+                        List<MetaMethod> arr = map.computeIfAbsent(paramTypes[0], k -> new ArrayList<MetaMethod>(4));
                         if (useInstanceMethods) {
                             final NewInstanceMetaMethod metaMethod = new NewInstanceMetaMethod(method);
                             arr.add(metaMethod);
@@ -254,16 +245,12 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
 
     private void createMetaMethodFromClass(Map<CachedClass, List<MetaMethod>> map, Class aClass) {
         try {
-            MetaMethod method = (MetaMethod) aClass.newInstance();
+            MetaMethod method = (MetaMethod) aClass.getDeclaredConstructor().newInstance();
             final CachedClass declClass = method.getDeclaringClass();
-            List<MetaMethod> arr = map.get(declClass);
-            if (arr == null) {
-                arr = new ArrayList<MetaMethod>(4);
-                map.put(declClass, arr);
-            }
+            List<MetaMethod> arr = map.computeIfAbsent(declClass, k -> new ArrayList<MetaMethod>(4));
             arr.add(method);
             instanceMethods.add(method);
-        } catch (InstantiationException | IllegalAccessException e) { /* ignore */
+        } catch (InstantiationException | IllegalAccessException | NoSuchMethodException | InvocationTargetException e) { /* ignore */
         }
     }
 
@@ -277,12 +264,12 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
 
     /**
      * if oldMc is null, newMc will replace whatever meta class was used before.
-     * if oldMc is not null, then newMc will be used only if he stored mc is
+     * if oldMc is not null, then newMc will be used only if the stored mc is
      * the same as oldMc
      */
     private void setMetaClass(Class theClass, MetaClass oldMc, MetaClass newMc) {
         final ClassInfo info = ClassInfo.getClassInfo(theClass);
-        
+
         MetaClass mc = null;
         info.lock();
         try {
@@ -295,11 +282,11 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
             fireConstantMetaClassUpdate(null, theClass, mc, newMc);
         }
     }
-    
+
     public void removeMetaClass(Class theClass) {
         setMetaClass(theClass, null, null);
     }
-    
+
     /**
      * Registers a new MetaClass in the registry to customize the type
      *
@@ -323,7 +310,7 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
         finally {
             info.unlock();
         }
-        
+
         fireConstantMetaClassUpdate(obj, theClass, oldMC, theMetaClass);
     }
 
@@ -333,16 +320,16 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
     }
 
     private volatile MetaClassCreationHandle metaClassCreationHandle = new MetaClassCreationHandle();
-    
+
     /**
      * Gets a handle internally used to create MetaClass implementations
      * WARNING: experimental code, likely to change soon
      * @return the handle
-     */    
+     */
     public MetaClassCreationHandle getMetaClassCreationHandler() {
         return metaClassCreationHandle;
     }
-    
+
     /**
      * Sets a handle internally used to create MetaClass implementations.
      * When replacing the handle with a custom version, you should
@@ -356,7 +343,7 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
         ClassInfo.clearModifiedExpandos();
         handle.setDisableCustomMetaClassLookup(metaClassCreationHandle.isDisableCustomMetaClassLookup());
         metaClassCreationHandle = handle;
-    }    
+    }
 
     /**
      * Adds a listener for constant meta classes.
@@ -367,7 +354,7 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
             changeListenerList.add(listener);
         }
     }
-    
+
 
     /**
      * Adds a listener for constant meta classes. This listener cannot be removed!
@@ -402,13 +389,13 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
     protected void fireConstantMetaClassUpdate(Object obj, Class c, final MetaClass oldMC, MetaClass newMc) {
         MetaClassRegistryChangeEventListener[]  listener = getMetaClassRegistryChangeEventListeners();
         MetaClassRegistryChangeEvent cmcu = new MetaClassRegistryChangeEvent(this, obj, c, oldMC, newMc);
-        for (int i = 0; i<listener.length; i++) {
-            listener[i].updateConstantMetaClass(cmcu);
+        for (MetaClassRegistryChangeEventListener metaClassRegistryChangeEventListener : listener) {
+            metaClassRegistryChangeEventListener.updateConstantMetaClass(cmcu);
         }
     }
 
     /**
-     * Gets an array of of all registered ConstantMetaClassListener instances.
+     * Gets an array of all registered ConstantMetaClassListener instances.
      */
     public MetaClassRegistryChangeEventListener[] getMetaClassRegistryChangeEventListeners() {
         synchronized (changeListenerList) {
@@ -419,9 +406,9 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
             return ret.toArray(EMPTY_METACLASSREGISTRYCHANGEEVENTLISTENER_ARRAY);
         }
     }
-    
+
     /**
-     * Singleton of MetaClassRegistry. 
+     * Singleton of MetaClassRegistry.
      *
      * @param includeExtension
      * @return the registry
@@ -447,21 +434,21 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
     public FastArray getStaticMethods() {
         return staticMethods;
     }
-    
+
     /**
      * Returns an iterator to iterate over all constant meta classes.
      * This iterator can be seen as making a snapshot of the current state
      * of the registry. The snapshot will include all meta classes that has
-     * been used unless they are already collected. Collected meta classes 
+     * been used unless they are already collected. Collected meta classes
      * will be skipped automatically, so you can expect that each element
      * of the iteration is not null. Calling this method is thread safe, the
      * usage of the iterator is not.
-     *  
+     *
      * @return the iterator.
-     */    
+     */
     public Iterator iterator() {
         final MetaClass[] refs = metaClassInfo.toArray(EMPTY_METACLASS_ARRAY);
-        
+
         return new Iterator() {
             // index in the ref array
             private int index = 0;
@@ -484,19 +471,19 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
                 }
                 return hasNext;
             }
-            
+
             private void ensureNext() {
-                // we ensure that hasNext has been called before 
+                // we ensure that hasNext has been called before
                 // next is called
                 hasNext();
                 hasNextCalled = false;
             }
-            
+
             public Object next() {
                 ensureNext();
                 return currentMeta;
             }
-            
+
             public void remove() {
                 ensureNext();
                 setMetaClass(currentMeta.getTheClass(), currentMeta, null);
@@ -528,11 +515,7 @@ public class MetaClassRegistryImpl implements MetaClassRegistry{
             List<MetaMethod> metaMethods = module.getMetaMethods();
             for (MetaMethod metaMethod : metaMethods) {
                 CachedClass cachedClass = metaMethod.getDeclaringClass();
-                List<MetaMethod> methods = map.get(cachedClass);
-                if (methods == null) {
-                    methods = new ArrayList<MetaMethod>(4);
-                    map.put(cachedClass, methods);
-                }
+                List<MetaMethod> methods = map.computeIfAbsent(cachedClass, k -> new ArrayList<MetaMethod>(4));
                 methods.add(metaMethod);
                 if (metaMethod.isStatic()) {
                     staticMethods.add(metaMethod);
