@@ -35,12 +35,12 @@ class ArtifactRequestBuilder
 
     private final RequestTrace trace;
 
-    private List<ArtifactRequest> requests;
+    private final List<ArtifactRequest> requests;
 
     ArtifactRequestBuilder( RequestTrace trace )
     {
         this.trace = trace;
-        this.requests = new ArrayList<ArtifactRequest>();
+        this.requests = new ArrayList<>();
     }
 
     public List<ArtifactRequest> getRequests()
