@@ -1,20 +1,30 @@
-/*******************************************************************************
- * Copyright (c) 2010, 2014 Sonatype, Inc.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
- * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
- *
- * Contributors:
- *    Sonatype, Inc. - initial API and implementation
- *******************************************************************************/
 package org.eclipse.aether.util.repository;
 
+/*
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ * 
+ *  http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
+ */
+
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import static java.util.Objects.requireNonNull;
 import java.util.StringTokenizer;
 import java.util.regex.Pattern;
 
@@ -29,7 +39,7 @@ public final class DefaultProxySelector
     implements ProxySelector
 {
 
-    private List<ProxyDef> proxies = new ArrayList<ProxyDef>();
+    private final List<ProxyDef> proxies = new ArrayList<>();
 
     /**
      * Adds the specified proxy definition to the selector. Proxy definitions are ordered, the first matching proxy for
@@ -41,10 +51,7 @@ public final class DefaultProxySelector
      */
     public DefaultProxySelector add( Proxy proxy, String nonProxyHosts )
     {
-        if ( proxy == null )
-        {
-            throw new IllegalArgumentException( "proxy not specified" );
-        }
+        requireNonNull( proxy, "proxy cannot be null" );
         proxies.add( new ProxyDef( proxy, nonProxyHosts ) );
 
         return this;
@@ -52,7 +59,8 @@ public final class DefaultProxySelector
 
     public Proxy getProxy( RemoteRepository repository )
     {
-        Map<String, ProxyDef> candidates = new HashMap<String, ProxyDef>();
+        requireNonNull( repository, "repository cannot be null" );
+        Map<String, ProxyDef> candidates = new HashMap<>();
 
         String host = repository.getHost();
         for ( ProxyDef proxy : proxies )
@@ -96,19 +104,20 @@ public final class DefaultProxySelector
 
         private final Pattern[] patterns;
 
-        public NonProxyHosts( String nonProxyHosts )
+        NonProxyHosts( String nonProxyHosts )
         {
-            List<Pattern> patterns = new ArrayList<Pattern>();
+            List<Pattern> patterns = new ArrayList<>();
             if ( nonProxyHosts != null )
             {
-                for ( StringTokenizer tokenizer = new StringTokenizer( nonProxyHosts, "|" ); tokenizer.hasMoreTokens(); )
+                for ( StringTokenizer tokenizer = new StringTokenizer( nonProxyHosts, "|" );
+                      tokenizer.hasMoreTokens(); )
                 {
                     String pattern = tokenizer.nextToken();
                     pattern = pattern.replace( ".", "\\." ).replace( "*", ".*" );
                     patterns.add( Pattern.compile( pattern, Pattern.CASE_INSENSITIVE ) );
                 }
             }
-            this.patterns = patterns.toArray( new Pattern[patterns.size()] );
+            this.patterns = patterns.toArray( new Pattern[0] );
         }
 
         boolean isNonProxyHost( String host )
@@ -135,7 +144,7 @@ public final class DefaultProxySelector
 
         final NonProxyHosts nonProxyHosts;
 
-        public ProxyDef( Proxy proxy, String nonProxyHosts )
+        ProxyDef( Proxy proxy, String nonProxyHosts )
         {
             this.proxy = proxy;
             this.nonProxyHosts = new NonProxyHosts( nonProxyHosts );
