@@ -16,17 +16,22 @@
 package org.gradle.api.artifacts;
 
 import groovy.lang.Closure;
+import groovy.lang.DelegatesTo;
 import org.gradle.api.Action;
 import org.gradle.api.Incubating;
+import org.gradle.api.attributes.HasConfigurableAttributes;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.specs.Spec;
 import org.gradle.api.tasks.TaskDependency;
 import org.gradle.internal.HasInternalProtocol;
 
+import javax.annotation.Nullable;
 import java.io.File;
 import java.util.Map;
 import java.util.Set;
 
+import static groovy.lang.Closure.DELEGATE_FIRST;
+
 /**
  * A {@code Configuration} represents a group of artifacts and their dependencies.
  * Find more information about declaring dependencies to a configuration
@@ -37,10 +42,11 @@ import java.util.Set;
  * If you want to refer to the artifacts declared in this configuration
  * please use {@link #getArtifacts()} or {@link #getAllArtifacts()}.
  * Read more about declaring artifacts in the configuration in docs for {@link org.gradle.api.artifacts.dsl.ArtifactHandler}
- * <p>
+ *
+ * Please see the <a href="https://docs.gradle.org/current/userguide/declaring_dependencies.html" target="_top">Declaring Dependencies</a> User Manual chapter for more information.
  */
 @HasInternalProtocol
-public interface Configuration extends FileCollection {
+public interface Configuration extends FileCollection, HasConfigurableAttributes<Configuration> {
 
     /**
      * Returns the resolution strategy used by this configuration.
@@ -60,7 +66,17 @@ public interface Configuration extends FileCollection {
      * @return this configuration instance
      * @since 1.0-milestone-6
      */
-    Configuration resolutionStrategy(Closure closure);
+    Configuration resolutionStrategy(@DelegatesTo(value = ResolutionStrategy.class, strategy = DELEGATE_FIRST) Closure closure);
+
+    /**
+     * The resolution strategy provides extra details on how to resolve this configuration.
+     * See docs for {@link ResolutionStrategy} for more info and examples.
+     *
+     * @param action action applied to the {@link ResolutionStrategy}
+     * @return this configuration instance
+     * @since 3.1
+     */
+    Configuration resolutionStrategy(Action<? super ResolutionStrategy> action);
 
     /**
      * The states a configuration can be into. A configuration is only mutable as long as it is
@@ -86,7 +102,8 @@ public interface Configuration extends FileCollection {
     /**
      * A {@link org.gradle.api.Namer} namer for configurations that returns {@link #getName()}.
      */
-    static class Namer implements org.gradle.api.Namer<Configuration> {
+    class Namer implements org.gradle.api.Namer<Configuration> {
+        @Override
         public String determineName(Configuration c) {
             return c.getName();
         }
@@ -156,6 +173,7 @@ public interface Configuration extends FileCollection {
      *
      * @return the description. May be null.
      */
+    @Nullable
     String getDescription();
 
     /**
@@ -164,7 +182,7 @@ public interface Configuration extends FileCollection {
      * @param description the description. May be null
      * @return this configuration
      */
-    Configuration setDescription(String description);
+    Configuration setDescription(@Nullable String description);
 
     /**
      * Gets a ordered set including this configuration and all superconfigurations
@@ -182,7 +200,7 @@ public interface Configuration extends FileCollection {
     Set<File> resolve();
 
     /**
-     * Takes a closure which gets coerced into a Spec. Behaves otherwise in the same way as
+     * Takes a closure which gets coerced into a {@link Spec}. Behaves otherwise in the same way as
      * {@link #files(org.gradle.api.specs.Spec)}.
      *
      * @param dependencySpecClosure The closure describing a filter applied to the all the dependencies of this configuration (including dependencies from extended configurations).
@@ -205,13 +223,13 @@ public interface Configuration extends FileCollection {
      * But only the resulting set of files belonging to the specified dependencies
      * is returned.
      *
-     * @param dependencies The dependences to be resolved
+     * @param dependencies The dependencies to be resolved
      * @return The files of a subset of dependencies of this configuration.
      */
     Set<File> files(Dependency... dependencies);
 
     /**
-     * Resolves this configuration lazily. The resolve happens when the elements of the returned FileCollection get accessed the first time.
+     * Resolves this configuration lazily. The resolve happens when the elements of the returned {@link FileCollection} get accessed the first time.
      * This locates and downloads the files which make up this configuration. Only the resulting set of files belonging to the subset
      * of dependencies specified by the dependencySpec is contained in the FileCollection.
      *
@@ -221,7 +239,7 @@ public interface Configuration extends FileCollection {
     FileCollection fileCollection(Spec<? super Dependency> dependencySpec);
 
     /**
-     * Takes a closure which gets coerced into a Spec. Behaves otherwise in the same way as
+     * Takes a closure which gets coerced into a {@link Spec}. Behaves otherwise in the same way as
      * {@link #fileCollection(org.gradle.api.specs.Spec)}.
      *
      * @param dependencySpecClosure The closure describing a filter applied to the all the dependencies of this configuration (including dependencies from extended configurations).
@@ -230,7 +248,7 @@ public interface Configuration extends FileCollection {
     FileCollection fileCollection(Closure dependencySpecClosure);
 
     /**
-     * Resolves this configuration lazily. The resolve happens when the elements of the returned FileCollection get accessed the first time.
+     * Resolves this configuration lazily. The resolve happens when the elements of the returned {@link FileCollection} get accessed the first time.
      * This locates and downloads the files which make up this configuration. Only the resulting set of files belonging to specified
      * dependencies is contained in the FileCollection.
      *
@@ -239,10 +257,9 @@ public interface Configuration extends FileCollection {
      */
     FileCollection fileCollection(Dependency... dependencies);
 
-
     /**
      * Resolves this configuration. This locates and downloads the files which make up this configuration, and returns
-     * a ResolvedConfiguration that may be used to determine information about the resolve (including errors).
+     * a {@link ResolvedConfiguration} that may be used to determine information about the resolve (including errors).
      *
      * @return The ResolvedConfiguration object
      */
@@ -258,11 +275,12 @@ public interface Configuration extends FileCollection {
     String getUploadTaskName();
 
     /**
-     * Returns a {@code TaskDependency} object containing all required dependencies to build the internal dependencies
+     * Returns a {@code TaskDependency} object containing all required dependencies to build the local dependencies
      * (e.g.<!-- --> project dependencies) belonging to this configuration or to one of its super configurations.
      *
      * @return a TaskDependency object
      */
+    @Override
     TaskDependency getBuildDependencies();
 
     /**
@@ -279,21 +297,49 @@ public interface Configuration extends FileCollection {
     TaskDependency getTaskDependencyFromProjectDependency(boolean useDependedOn, final String taskName);
 
     /**
-     * Gets the set of dependencies directly contained in this configuration
+     * Gets the set of declared dependencies directly contained in this configuration
      * (ignoring superconfigurations).
+     * <p>
+     * This method does not resolve the configuration. Therefore, the return value does not include
+     * transitive dependencies.
      *
      * @return the set of dependencies
+     * @see #extendsFrom(Configuration...)
      */
     DependencySet getDependencies();
 
     /**
-     * <p>Gets the complete set of dependencies including those contributed by
-     * superconfigurations.</p>
+     * Gets the complete set of declared dependencies including those contributed by
+     * superconfigurations.
+     * <p>
+     * This method does not resolve the configuration. Therefore, the return value does not include
+     * transitive dependencies.
      *
      * @return the (read-only) set of dependencies
+     * @see #extendsFrom(Configuration...)
      */
     DependencySet getAllDependencies();
 
+    /**
+     * Gets the set of dependency constraints directly contained in this configuration
+     * (ignoring superconfigurations).
+     *
+     * @return the set of dependency constraints
+     *
+     * @since 4.6
+     */
+    DependencyConstraintSet getDependencyConstraints();
+
+    /**
+     * <p>Gets the complete set of dependency constraints including those contributed by
+     * superconfigurations.</p>
+     *
+     * @return the (read-only) set of dependency constraints
+     *
+     * @since 4.6
+     */
+    DependencyConstraintSet getAllDependencyConstraints();
+
     /**
      * Returns the artifacts of this configuration excluding the artifacts of extended configurations.
      *
@@ -335,11 +381,10 @@ public interface Configuration extends FileCollection {
      *     <li>Another {@link Configuration} that references this one as a project dependency is resolved</li>
      * </ul>
      *
-     *
      * This method is useful for specifying default dependencies for a configuration:
-     * <pre autoTested='true'>
+     * <pre class='autoTested'>
      * configurations { conf }
-     * configurations['conf'].defaultDependencies { dependencies ->
+     * configurations['conf'].defaultDependencies { dependencies -&gt;
      *      dependencies.add(owner.project.dependencies.create("org.gradle:my-util:1.0"))
      * }
      * </pre>
@@ -352,9 +397,38 @@ public interface Configuration extends FileCollection {
      * @param action the action to execute when the configuration has no defined dependencies.
      * @return this
      */
-    @Incubating
     Configuration defaultDependencies(Action<? super DependencySet> action);
 
+    /**
+     * Execute the given action before the configuration first participates in
+     * dependency resolution. A {@code Configuration} will participate in dependency resolution
+     * when:
+     * <ul>
+     *     <li>The {@link Configuration} itself is resolved</li>
+     *     <li>Another {@link Configuration} that extends this one is resolved</li>
+     *     <li>Another {@link Configuration} that references this one as a project dependency is resolved</li>
+     * </ul>
+     *
+     * This method is useful for mutating the dependencies for a configuration:
+     * <pre class='autoTested'>
+     * configurations { conf }
+     * configurations['conf'].withDependencies { dependencies -&gt;
+     *      dependencies.each { dependency -&gt;
+     *          if (dependency.version == null) {
+     *              dependency.version { require '1.0' }
+     *          }
+     *      }
+     * }
+     * </pre>
+     *
+     * Actions will be executed in the order provided.
+     *
+     * @since 4.4
+     * @param action a dependency action to execute before the configuration is used.
+     * @return this
+     */
+    Configuration withDependencies(Action<? super DependencySet> action);
+
     /**
      * Returns all the configurations belonging to the same configuration container as this
      * configuration (including this configuration).
@@ -370,6 +444,22 @@ public interface Configuration extends FileCollection {
      */
     ResolvableDependencies getIncoming();
 
+    /**
+     * Returns the outgoing artifacts of this configuration.
+     *
+     * @return The outgoing artifacts of this configuration.
+     * @since 3.4
+     */
+    ConfigurationPublications getOutgoing();
+
+    /**
+     * Configures the outgoing artifacts of this configuration.
+     *
+     * @param action The action to perform the configuration.
+     * @since 3.4
+     */
+    void outgoing(Action<? super ConfigurationPublications> action);
+
     /**
      * Creates a copy of this configuration that only contains the dependencies directly in this configuration
      * (without contributions from superconfigurations).  The new configuration will be in the
@@ -407,7 +497,7 @@ public interface Configuration extends FileCollection {
     Configuration copyRecursive(Spec<? super Dependency> dependencySpec);
 
     /**
-     * Takes a closure which gets coerced into a Spec. Behaves otherwise in the same way as {@link #copy(org.gradle.api.specs.Spec)}
+     * Takes a closure which gets coerced into a {@link Spec}. Behaves otherwise in the same way as {@link #copy(org.gradle.api.specs.Spec)}
      *
      * @param dependencySpec filtering requirements
      * @return copy of this configuration
@@ -415,10 +505,60 @@ public interface Configuration extends FileCollection {
     Configuration copy(Closure dependencySpec);
 
     /**
-     * Takes a closure which gets coerced into a Spec. Behaves otherwise in the same way as {@link #copyRecursive(org.gradle.api.specs.Spec)}
+     * Takes a closure which gets coerced into a {@link Spec}. Behaves otherwise in the same way as {@link #copyRecursive(org.gradle.api.specs.Spec)}
      *
      * @param dependencySpec filtering requirements
      * @return copy of this configuration
      */
     Configuration copyRecursive(Closure dependencySpec);
+
+    /**
+     * Configures if a configuration can be consumed.
+     *
+     * @since 3.3
+     */
+    void setCanBeConsumed(boolean allowed);
+
+    /**
+     * Returns true if this configuration can be consumed from another project, or published. Defaults to true.
+     * @return true if this configuration can be consumed or published.
+     * @since 3.3
+     */
+    boolean isCanBeConsumed();
+
+    /**
+     * Configures if a configuration can be resolved.
+     *
+     * @since 3.3
+     */
+    void setCanBeResolved(boolean allowed);
+
+    /**
+     * Returns true if it is allowed to query or resolve this configuration. Defaults to true.
+     * @return true if this configuration can be queried or resolved.
+     * @since 3.3
+     */
+    boolean isCanBeResolved();
+
+    /**
+     * Tells that this configuration, when resolved, should resolve versions consistently
+     * from the resolution result of another resolvable configuration. For example, it's
+     * expected that the versions of the runtime classpath are the same as the versions
+     * from the compile classpath.
+     *
+     * @param versionsSource another resolvable configuration to use as reference for versions
+     * @return this configuration
+     *
+     * @since 6.8
+     */
+    @Incubating
+    Configuration shouldResolveConsistentlyWith(Configuration versionsSource);
+
+    /**
+     * Disables consistent resolution for this configuration
+     *
+     * @since 6.8
+     */
+    @Incubating
+    Configuration disableConsistentResolution();
 }
