@@ -24,12 +24,10 @@ import org.gradle.api.file.FileVisitDetails;
 import org.gradle.api.file.FileVisitor;
 import org.gradle.api.file.RelativePath;
 import org.gradle.api.internal.file.AbstractFileTreeElement;
-import org.gradle.api.internal.file.FileSystemSubset;
 import org.gradle.api.internal.file.collections.DirectoryFileTree;
-import org.gradle.api.internal.file.collections.FileSystemMirroringFileTree;
-import org.gradle.api.internal.file.collections.MinimalFileTree;
-import org.gradle.internal.hash.HashUtil;
-import org.gradle.internal.nativeintegration.filesystem.Chmod;
+import org.gradle.api.internal.file.collections.DirectoryFileTreeFactory;
+import org.gradle.internal.file.Chmod;
+import org.gradle.internal.hash.FileHasher;
 import org.gradle.internal.nativeintegration.filesystem.FileSystem;
 
 import java.io.File;
@@ -41,26 +39,37 @@ import java.util.Map;
 import java.util.TreeMap;
 import java.util.concurrent.atomic.AtomicBoolean;
 
-public class ZipFileTree implements MinimalFileTree, FileSystemMirroringFileTree {
+public class ZipFileTree extends AbstractArchiveFileTree {
     private final File zipFile;
-    private final Chmod chmod;
     private final File tmpDir;
+    private final Chmod chmod;
+    private final DirectoryFileTreeFactory directoryFileTreeFactory;
+    private final FileHasher fileHasher;
 
-    public ZipFileTree(File zipFile, File tmpDir, Chmod chmod) {
+    public ZipFileTree(File zipFile, File tmpDir, Chmod chmod, DirectoryFileTreeFactory directoryFileTreeFactory, FileHasher fileHasher) {
         this.zipFile = zipFile;
+        this.tmpDir = tmpDir;
         this.chmod = chmod;
-        String expandDirName = String.format("%s_%s", zipFile.getName(), HashUtil.createCompactMD5(zipFile.getAbsolutePath()));
-        this.tmpDir = new File(tmpDir, expandDirName);
+        this.directoryFileTreeFactory = directoryFileTreeFactory;
+        this.fileHasher = fileHasher;
     }
 
+    @Override
+    public String toString() {
+        return getDisplayName();
+    }
+
+    @Override
     public String getDisplayName() {
         return String.format("ZIP '%s'", zipFile);
     }
 
+    @Override
     public DirectoryFileTree getMirror() {
-        return new DirectoryFileTree(tmpDir);
+        return directoryFileTreeFactory.create(getExpandedDir());
     }
 
+    @Override
     public void visit(FileVisitor visitor) {
         if (!zipFile.exists()) {
             throw new InvalidUserDataException(String.format("Cannot expand %s as it does not exist.", getDisplayName()));
@@ -73,6 +82,7 @@ public class ZipFileTree implements MinimalFileTree, FileSystemMirroringFileTree
 
         try {
             ZipFile zip = new ZipFile(zipFile);
+            File expandedDir = getExpandedDir();
             try {
                 // The iteration order of zip.getEntries() is based on the hash of the zip entry. This isn't much use
                 // to us. So, collect the entries in a map and iterate over them in alphabetical order.
@@ -86,9 +96,9 @@ public class ZipFileTree implements MinimalFileTree, FileSystemMirroringFileTree
                 while (!stopFlag.get() && sortedEntries.hasNext()) {
                     ZipEntry entry = sortedEntries.next();
                     if (entry.isDirectory()) {
-                        visitor.visitDir(new DetailsImpl(entry, zip, stopFlag, chmod));
+                        visitor.visitDir(new DetailsImpl(zipFile, expandedDir, entry, zip, stopFlag, chmod));
                     } else {
-                        visitor.visitFile(new DetailsImpl(entry, zip, stopFlag, chmod));
+                        visitor.visitFile(new DetailsImpl(zipFile, expandedDir, entry, zip, stopFlag, chmod));
                     }
                 }
             } finally {
@@ -99,47 +109,70 @@ public class ZipFileTree implements MinimalFileTree, FileSystemMirroringFileTree
         }
     }
 
-    private class DetailsImpl extends AbstractFileTreeElement implements FileVisitDetails {
+    @Override
+    public File getBackingFile() {
+        return zipFile;
+    }
+
+    private File getExpandedDir() {
+        String expandedDirName = zipFile.getName() + "_" + fileHasher.hash(zipFile);
+        return new File(tmpDir, expandedDirName);
+    }
+
+    private static class DetailsImpl extends AbstractFileTreeElement implements FileVisitDetails {
+        private final File originalFile;
+        private final File expandedDir;
         private final ZipEntry entry;
         private final ZipFile zip;
         private final AtomicBoolean stopFlag;
         private File file;
 
-        public DetailsImpl(ZipEntry entry, ZipFile zip, AtomicBoolean stopFlag, Chmod chmod) {
+        public DetailsImpl(File originalFile, File expandedDir, ZipEntry entry, ZipFile zip, AtomicBoolean stopFlag, Chmod chmod) {
             super(chmod);
+            this.originalFile = originalFile;
+            this.expandedDir = expandedDir;
             this.entry = entry;
             this.zip = zip;
             this.stopFlag = stopFlag;
         }
 
+        @Override
         public String getDisplayName() {
-            return String.format("zip entry %s!%s", zipFile, entry.getName());
+            return String.format("zip entry %s!%s", originalFile, entry.getName());
         }
 
+        @Override
         public void stopVisiting() {
             stopFlag.set(true);
         }
 
+        @Override
         public File getFile() {
             if (file == null) {
-                file = new File(tmpDir, entry.getName());
-                copyTo(file);
+                file = new File(expandedDir, entry.getName());
+                if (!file.exists()) {
+                    copyTo(file);
+                }
             }
             return file;
         }
 
+        @Override
         public long getLastModified() {
             return entry.getTime();
         }
 
+        @Override
         public boolean isDirectory() {
             return entry.isDirectory();
         }
 
+        @Override
         public long getSize() {
             return entry.getSize();
         }
 
+        @Override
         public InputStream open() {
             try {
                 return zip.getInputStream(entry);
@@ -148,10 +181,12 @@ public class ZipFileTree implements MinimalFileTree, FileSystemMirroringFileTree
             }
         }
 
+        @Override
         public RelativePath getRelativePath() {
             return new RelativePath(!entry.isDirectory(), entry.getName().split("/"));
         }
 
+        @Override
         public int getMode() {
             int unixMode = entry.getUnixMode() & 0777;
             if (unixMode == 0) {
@@ -165,9 +200,4 @@ public class ZipFileTree implements MinimalFileTree, FileSystemMirroringFileTree
             return unixMode;
         }
     }
-
-    @Override
-    public void registerWatchPoints(FileSystemSubset.Builder builder) {
-        builder.add(zipFile);
-    }
 }
