@@ -29,6 +29,7 @@ import org.gradle.api.NonNullApi;
 import org.gradle.api.Project;
 import org.gradle.api.Task;
 import org.gradle.api.UnknownTaskException;
+import org.gradle.api.internal.CollectionCallbackActionDecorator;
 import org.gradle.api.internal.MutationGuards;
 import org.gradle.api.internal.NamedDomainObjectContainerConfigureDelegate;
 import org.gradle.api.internal.TaskInternal;
@@ -59,7 +60,6 @@ import org.gradle.model.internal.core.MutableModelNode;
 import org.gradle.model.internal.core.NamedEntityInstantiator;
 import org.gradle.model.internal.type.ModelType;
 import org.gradle.util.ConfigureUtil;
-import org.gradle.util.DeprecationLogger;
 import org.gradle.util.GUtil;
 
 import javax.annotation.Nullable;
@@ -95,8 +95,15 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
 
     private MutableModelNode modelNode;
 
-    public DefaultTaskContainer(final ProjectInternal project, Instantiator instantiator, final ITaskFactory taskFactory, ProjectAccessListener projectAccessListener, TaskStatistics statistics, BuildOperationExecutor buildOperationExecutor, CrossProjectConfigurator crossProjectConfigurator) {
-        super(Task.class, instantiator, project, MutationGuards.of(crossProjectConfigurator));
+    public DefaultTaskContainer(final ProjectInternal project,
+                                Instantiator instantiator,
+                                final ITaskFactory taskFactory,
+                                ProjectAccessListener projectAccessListener,
+                                TaskStatistics statistics,
+                                BuildOperationExecutor buildOperationExecutor,
+                                CrossProjectConfigurator crossProjectConfigurator,
+                                CollectionCallbackActionDecorator callbackDecorator) {
+        super(Task.class, instantiator, project, MutationGuards.of(crossProjectConfigurator), callbackDecorator);
         this.taskFactory = taskFactory;
         taskInstantiator = new TaskInstantiator(taskFactory, project);
         this.projectAccessListener = projectAccessListener;
@@ -105,6 +112,7 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         this.buildOperationExecutor = buildOperationExecutor;
     }
 
+    @Override
     public Task create(Map<String, ?> options) {
         assertMutable("create(Map<String, ?>)");
         return doCreate(options, Actions.doNothing());
@@ -223,9 +231,7 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         if (replaceExisting) {
             Task existing = findByNameWithoutRules(name);
             if (existing != null) {
-                DeprecationLogger.nagUserOfDeprecated("Replacing a task that may have been used by other plugins",
-                    "Use a different name for this task ('" + name + "') or avoid creating the original task you are trying to replace.");
-                removeInternal(existing);
+                throw new IllegalStateException("Replacing an existing task that may have already been used by other plugins is not supported.  Use a different name for this task ('" + name + "').");
             } else {
                 TaskCreatingProvider<? extends Task> taskProvider = Cast.uncheckedCast(findByNameLaterWithoutRules(name));
                 if (taskProvider != null) {
@@ -233,10 +239,7 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
 
                     final Action<? super T> onCreate;
                     if (!taskProvider.getType().isAssignableFrom(task.getClass())) {
-                        DeprecationLogger.nagUserOfDeprecated(
-                            "Replacing an existing task with an incompatible type",
-                            "Use a different name for this task ('" + name + "'), use a compatible type (" + ((TaskInternal)task).getTaskIdentity().type.getName() + ") or avoid creating the original task you are trying to replace.");
-                        onCreate = getEventRegister().getAddActions();
+                        throw new IllegalStateException("Replacing an existing task with an incompatible type is not supported.  Use a different name for this task ('" + name + "') or use a compatible type (" + ((TaskInternal) task).getTaskIdentity().type.getName() + ")");
                     } else {
                         onCreate = Cast.uncheckedCast(taskProvider.getOnCreateActions().mergeFrom(getEventRegister().getAddActions()));
                     }
@@ -244,9 +247,7 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
                     add(task, onCreate);
                     return; // Exit early as we are reusing the create actions from the provider
                 } else {
-                    DeprecationLogger.nagUserOfDeprecated(
-                        "Unnecessarily replacing a task that does not exist",
-                        "Try using create() or register() directly instead. You attempted to replace a task named '" + name + "', but no task exists with that name already.");
+                    throw new IllegalStateException("Unnecessarily replacing a task that does not exist is not supported.  Use create() or register() directly instead.  You attempted to replace a task named '" + name + "', but there is no existing task with that name.");
                 }
             }
         } else if (hasWithName(name)) {
@@ -260,6 +261,7 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         throw new DuplicateTaskException(String.format("Cannot add task '%s' as a task with that name already exists.", task));
     }
 
+    @Override
     public <U extends Task> U maybeCreate(String name, Class<U> type) throws InvalidUserDataException {
         Task existing = findByName(name);
         if (existing != null) {
@@ -268,6 +270,7 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         return create(name, type);
     }
 
+    @Override
     public Task create(Map<String, ?> options, Closure configureClosure) throws InvalidUserDataException {
         assertMutable("create(Map<String, ?>, Closure)");
         return doCreate(options, ConfigureUtil.configureUsing(configureClosure));
@@ -285,7 +288,10 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         return doCreate(name, type, constructorArgs, Actions.doNothing());
     }
 
-    private <T extends Task> T doCreate(final String name, final Class<T> type, final Object[] constructorArgs, final Action<? super T> configureAction) throws InvalidUserDataException {
+    /**
+     * @param constructorArgs null == do not invoke constructor, empty == invoke constructor with no args, non-empty = invoke constructor with args
+     */
+    private <T extends Task> T doCreate(final String name, final Class<T> type, @Nullable final Object[] constructorArgs, final Action<? super T> configureAction) throws InvalidUserDataException {
         final TaskIdentity<T> identity = TaskIdentity.create(name, type, project);
         return buildOperationExecutor.call(new CallableBuildOperation<T>() {
             @Override
@@ -309,25 +315,30 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         });
     }
 
-    private <T extends Task> T createTask(TaskIdentity<T> identity, Object[] constructorArgs) throws InvalidUserDataException {
-        for (int i = 0; i < constructorArgs.length; i++) {
-            if (constructorArgs[i] == null) {
-                throw new NullPointerException(String.format("Received null for %s constructor argument #%s", identity.type.getName(), i + 1));
+    private <T extends Task> T createTask(TaskIdentity<T> identity, @Nullable Object[] constructorArgs) throws InvalidUserDataException {
+        if (constructorArgs != null) {
+            for (int i = 0; i < constructorArgs.length; i++) {
+                if (constructorArgs[i] == null) {
+                    throw new NullPointerException(String.format("Received null for %s constructor argument #%s", identity.type.getName(), i + 1));
+                }
             }
         }
         return taskFactory.create(identity, constructorArgs);
     }
 
+    @Override
     public Task create(String name) {
         assertMutable("create(String)");
         return doCreate(name, DefaultTask.class, NO_ARGS, Actions.doNothing());
     }
 
+    @Override
     public Task create(String name, Action<? super Task> configureAction) throws InvalidUserDataException {
         assertMutable("create(String, Action)");
         return doCreate(name, DefaultTask.class, NO_ARGS, configureAction);
     }
 
+    @Override
     public Task maybeCreate(String name) {
         Task task = findByName(name);
         if (task != null) {
@@ -336,16 +347,19 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         return create(name);
     }
 
+    @Override
     public Task replace(String name) {
         assertMutable("replace(String)");
         return replace(name, DefaultTask.class);
     }
 
+    @Override
     public Task create(String name, Closure configureClosure) {
         assertMutable("create(String, Closure)");
         return doCreate(name, DefaultTask.class, NO_ARGS, ConfigureUtil.configureUsing(configureClosure));
     }
 
+    @Override
     public <T extends Task> T create(String name, Class<T> type, Action<? super T> configuration) throws InvalidUserDataException {
         assertMutable("create(String, Class, Action)");
         T task = create(name, type);
@@ -387,7 +401,9 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         if (hasWithName(name)) {
             failOnDuplicateTask(name);
         }
+
         final TaskIdentity<T> identity = TaskIdentity.create(name, type, project);
+
         TaskProvider<T> provider = buildOperationExecutor.call(new CallableBuildOperation<TaskProvider<T>>() {
             @Override
             public BuildOperationDescriptor.Builder description() {
@@ -396,8 +412,10 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
 
             @Override
             public TaskProvider<T> call(BuildOperationContext context) {
-                TaskProvider<T> provider = Cast.uncheckedCast(getInstantiator()
-                    .newInstance(TaskCreatingProvider.class, DefaultTaskContainer.this, identity, configurationAction, constructorArgs)
+                TaskProvider<T> provider = Cast.uncheckedNonnullCast(
+                    getInstantiator().newInstance(
+                        TaskCreatingProvider.class, DefaultTaskContainer.this, identity, configurationAction, constructorArgs
+                    )
                 );
                 addLaterInternal(provider);
                 context.setResult(REGISTER_RESULT);
@@ -412,6 +430,7 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         return provider;
     }
 
+    @Override
     public <T extends Task> T replace(final String name, final Class<T> type) {
         assertMutable("replace(String, Class)");
         final TaskIdentity<T> identity = TaskIdentity.create(name, type, project);
@@ -419,7 +438,7 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
             @Override
             public T call(BuildOperationContext context) {
                 try {
-                    T task = taskFactory.create(identity);
+                    T task = taskFactory.create(identity, NO_ARGS);
                     addTask(task, true);
                     context.setResult(REALIZE_RESULT);
                     return task;
@@ -435,6 +454,13 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         });
     }
 
+    @Override
+    public <T extends Task> T createWithoutConstructor(String name, Class<T> type) {
+        assertMutable("createWithoutConstructor(String, Class, Object...)");
+        return doCreate(name, type, null, Actions.doNothing());
+    }
+
+    @Override
     public Task findByPath(String path) {
         if (Strings.isNullOrEmpty(path)) {
             throw new InvalidUserDataException("A path must be specified!");
@@ -453,6 +479,7 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         return project.getTasks().findByName(StringUtils.substringAfterLast(path, Project.PATH_SEPARATOR));
     }
 
+    @Override
     public Task resolveTask(String path) {
         if (Strings.isNullOrEmpty(path)) {
             throw new InvalidUserDataException("A path must be specified!");
@@ -460,6 +487,7 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         return getByPath(path);
     }
 
+    @Override
     public Task getByPath(String path) throws UnknownTaskException {
         Task task = findByPath(path);
         if (task == null) {
@@ -468,6 +496,7 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         return task;
     }
 
+    @Override
     public TaskContainerInternal configure(Closure configureClosure) {
         return ConfigureUtil.configureSelf(configureClosure, this, new NamedDomainObjectContainerConfigureDelegate(configureClosure, this));
     }
@@ -477,6 +506,7 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         return taskInstantiator;
     }
 
+    @Override
     public DynamicObject getTasksAsDynamicObject() {
         return getElementsAsDynamicObject();
     }
@@ -493,6 +523,7 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         }
     }
 
+    @Override
     public void realize() {
         if (modelNode != null) {
             project.getModelRegistry().realizeNode(modelNode.getPath());
@@ -526,6 +557,7 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         return false;
     }
 
+    @Override
     public Task findByName(String name) {
         Task task = super.findByName(name);
         if (task != null) {
@@ -541,10 +573,12 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         return project.getModelRegistry().atStateOrLater(taskPath, ModelType.of(Task.class), minState);
     }
 
+    @Override
     public <U extends Task> NamedDomainObjectContainer<U> containerWithType(Class<U> type) {
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public Set<? extends Class<? extends Task>> getCreateableTypes() {
         return Collections.singleton(getType());
     }
@@ -559,32 +593,30 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         return Cast.uncheckedCast(instantiator.newInstance(DefaultRealizableTaskCollection.class, type, super.withType(type), modelNode, instantiator));
     }
 
+    @Deprecated
     @Override
     public boolean remove(Object o) {
-        warnAboutRemoveMethodDeprecation("remove(Object)");
-        return removeInternal(o);
+        throw unsupportedTaskRemovalException();
     }
 
     private boolean removeInternal(Object o) {
         return super.remove(o);
     }
 
+    @Deprecated
     @Override
     public boolean removeAll(Collection<?> c) {
-        warnAboutRemoveMethodDeprecation("removeAll(Collection)");
-        return super.removeAll(c);
+        throw unsupportedTaskRemovalException();
     }
 
     @Override
     public void clear() {
-        warnAboutRemoveMethodDeprecation("clear()");
-        super.clear();
+        throw unsupportedTaskRemovalException();
     }
 
     @Override
     public boolean retainAll(Collection<?> target) {
-        warnAboutRemoveMethodDeprecation("retainAll(Collection)");
-        return super.retainAll(target);
+        throw unsupportedTaskRemovalException();
     }
 
     @Override
@@ -603,26 +635,23 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
 
             @Override
             public void remove() {
-                warnAboutRemoveMethodDeprecation("iterator()#remove()");
-                delegate.remove();
+                throw unsupportedTaskRemovalException();
             }
         };
     }
 
+    private static RuntimeException unsupportedTaskRemovalException() {
+        return new UnsupportedOperationException("Removing tasks from the task container is not supported.  Disable the tasks or use replace() instead.");
+    }
+
     @Override
     public Action<? super Task> whenObjectRemoved(Action<? super Task> action) {
-        warnAboutRemoveMethodDeprecation("whenObjectRemoved(Action)");
-        return super.whenObjectRemoved(action);
+        throw new UnsupportedOperationException("Registering actions on task removal is not supported.");
     }
 
     @Override
     public void whenObjectRemoved(Closure action) {
-        warnAboutRemoveMethodDeprecation("whenObjectRemoved(Closure)");
-        super.whenObjectRemoved(action);
-    }
-
-    private void warnAboutRemoveMethodDeprecation(String methodName) {
-        DeprecationLogger.nagUserOfDiscontinuedMethod("TaskContainer." + methodName, "Prefer disabling the task instead, see Task.setEnabled(boolean).");
+        throw new UnsupportedOperationException("Registering actions on task removal is not supported.");
     }
 
     // Cannot be private due to reflective instantiation
@@ -642,9 +671,8 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
         }
 
         @Override
-        public boolean maybeVisitBuildDependencies(TaskDependencyResolveContext context) {
-            context.add(get());
-            return true;
+        public ValueProducer getProducer() {
+            return ValueProducer.taskState(get());
         }
 
         @Override
@@ -704,15 +732,13 @@ public class DefaultTaskContainer extends DefaultTaskCollection<Task> implements
     @Deprecated
     @Override
     public boolean add(Task o) {
-        DeprecationLogger.nagUserOfReplacedMethodInvocation("TaskContainer.add()", "TaskContainer.register()");
-        return addInternal(o);
+        throw new UnsupportedOperationException("Adding a task directly to the task container is not supported.  Use register() instead.");
     }
 
     @Deprecated
     @Override
     public boolean addAll(Collection<? extends Task> c) {
-        DeprecationLogger.nagUserOfDiscontinuedMethodInvocation("TaskContainer.addAll()");
-        return addAllInternal(c);
+        throw new UnsupportedOperationException("Adding a collection of tasks directly to the task container is not supported.  Use register() instead.");
     }
 
     @Override
