@@ -17,7 +17,7 @@ package org.gradle.api.tasks;
 
 /**
  * Executes a command line process. Example:
- * <pre autoTested=''>
+ * <pre class='autoTested'>
  * task stopTomcat(type:Exec) {
  *   workingDir '../tomcat/bin'
  *
@@ -37,7 +37,7 @@ package org.gradle.api.tasks;
  * }
  * </pre>
  */
-public class Exec extends AbstractExecTask {
+public class Exec extends AbstractExecTask<Exec> {
     public Exec() {
         super(Exec.class);
     }
