@@ -15,9 +15,25 @@
  */
 package org.gradle.api.tasks;
 
+import org.gradle.api.Incubating;
+
 /**
  * Provides information about some work which was performed.
  */
 public interface WorkResult {
-    public boolean getDidWork();
+    boolean getDidWork();
+
+    /**
+     * Returns this result if it did work, otherwise returns the result given as a parameter.
+     *
+     * @since 6.0
+     */
+    @Incubating
+    default WorkResult or(WorkResult other) {
+        if (!getDidWork()) {
+            return other;
+        } else {
+            return this;
+        }
+    }
 }
