@@ -19,19 +19,24 @@ package org.gradle.cache.internal;
 import org.gradle.cache.CacheRepository;
 import org.gradle.util.GradleVersion;
 
+import javax.annotation.Nullable;
 import java.io.File;
 
 public class CacheRepositoryServices {
     private final File gradleUserHomeDir;
     private final File projectCacheDir;
 
-    public CacheRepositoryServices(File gradleUserHomeDir, File projectCacheDir) {
+    public CacheRepositoryServices(File gradleUserHomeDir, @Nullable File projectCacheDir) {
         this.gradleUserHomeDir = gradleUserHomeDir;
         this.projectCacheDir = projectCacheDir;
     }
 
-    protected CacheRepository createCacheRepository(CacheFactory factory) {
-        DefaultCacheScopeMapping scopeMapping = new DefaultCacheScopeMapping(gradleUserHomeDir, projectCacheDir, GradleVersion.current());
+    // We need to return `DefaultCacheScopeMapping` instead of `CacheScopeMapping` so the `GlobalCache` interface is picked up.
+    protected DefaultCacheScopeMapping createCacheScopeMapping() {
+        return new DefaultCacheScopeMapping(gradleUserHomeDir, projectCacheDir, GradleVersion.current());
+    }
+
+    protected CacheRepository createCacheRepository(CacheFactory factory, CacheScopeMapping scopeMapping) {
         return new DefaultCacheRepository(
             scopeMapping,
             factory);
