@@ -5,7 +5,7 @@
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ * https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -22,11 +22,14 @@ import io.micronaut.core.convert.ConversionContext;
 import io.micronaut.core.convert.ConversionService;
 import io.micronaut.core.convert.exceptions.ConversionErrorException;
 import io.micronaut.core.type.Argument;
+import io.micronaut.core.type.ArgumentCoercible;
 import io.micronaut.core.util.ArgumentUtils;
 
-import javax.annotation.Nonnull;
-import javax.annotation.Nullable;
+import io.micronaut.core.annotation.NonNull;
+import io.micronaut.core.annotation.Nullable;
 import javax.annotation.concurrent.Immutable;
+import java.util.Arrays;
+import java.util.Collection;
 import java.util.Optional;
 
 /**
@@ -43,12 +46,12 @@ import java.util.Optional;
  * @see BeanIntrospection
  */
 @Immutable
-public interface BeanProperty<B, T> extends AnnotatedElement, AnnotationMetadataDelegate {
+public interface BeanProperty<B, T> extends AnnotatedElement, AnnotationMetadataDelegate, ArgumentCoercible<T> {
 
     /**
      * @return The declaring bean introspection.
      */
-    @Nonnull BeanIntrospection<B> getDeclaringBean();
+    @NonNull BeanIntrospection<B> getDeclaringBean();
 
     /**
      * Read the bean value.
@@ -56,7 +59,7 @@ public interface BeanProperty<B, T> extends AnnotatedElement, AnnotationMetadata
      * @return The value
      * @throws IllegalArgumentException If the bean instance if not of the correct type
      */
-    @Nullable T get(@Nonnull B bean);
+    @Nullable T get(@NonNull B bean);
 
     /**
      * Read the value and try to convert it to the given type.
@@ -65,7 +68,7 @@ public interface BeanProperty<B, T> extends AnnotatedElement, AnnotationMetadata
      * @param <T2> The generic type
      * @return The value if conversion was possible.
      */
-    default @Nonnull <T2> Optional<T2> get(@Nonnull B bean, @Nonnull Class<T2> type) {
+    default @NonNull <T2> Optional<T2> get(@NonNull B bean, @NonNull Class<T2> type) {
         ArgumentUtils.requireNonNull("bean", bean);
         ArgumentUtils.requireNonNull("type", type);
         final Argument<T2> argument = Argument.of(type);
@@ -79,7 +82,7 @@ public interface BeanProperty<B, T> extends AnnotatedElement, AnnotationMetadata
      * @param <T2> The generic type
      * @return The value if conversion was possible.
      */
-    default <T2> Optional<T2> get(@Nonnull B bean, @Nonnull Argument<T2> argument) {
+    default <T2> Optional<T2> get(@NonNull B bean, @NonNull Argument<T2> argument) {
         ArgumentUtils.requireNonNull("bean", bean);
         ArgumentUtils.requireNonNull("type", argument);
 
@@ -94,7 +97,7 @@ public interface BeanProperty<B, T> extends AnnotatedElement, AnnotationMetadata
      * @param <T2> The generic type
      * @return The value if conversion was possible.
      */
-    default <T2> Optional<T2> get(@Nonnull B bean, @Nonnull ArgumentConversionContext<T2> conversionContext) {
+    default <T2> Optional<T2> get(@NonNull B bean, @NonNull ArgumentConversionContext<T2> conversionContext) {
         ArgumentUtils.requireNonNull("bean", bean);
         ArgumentUtils.requireNonNull("conversionContext", conversionContext);
 
@@ -110,7 +113,7 @@ public interface BeanProperty<B, T> extends AnnotatedElement, AnnotationMetadata
      * @param <T2> The generic type
      * @return The value if conversion was possible.
      */
-    default @Nullable <T2> T2 get(@Nonnull B bean, @Nonnull Class<T2> type, @Nullable T2 defaultValue) {
+    default @Nullable <T2> T2 get(@NonNull B bean, @NonNull Class<T2> type, @Nullable T2 defaultValue) {
         ArgumentUtils.requireNonNull("bean", bean);
         //noinspection ConstantConditions
         if (type == null) {
@@ -121,13 +124,97 @@ public interface BeanProperty<B, T> extends AnnotatedElement, AnnotationMetadata
         }
     }
 
+    /**
+     * This method returns true if the property can be mutated either via copy constructor or bean setter.
+     *
+     * @return True if it is mutable
+     * @see #withValue(Object, Object)
+     * @since 2.3.0
+     */
+    default boolean hasSetterOrConstructorArgument() {
+        BeanIntrospection<B> declaringBean = getDeclaringBean();
+        return !isReadOnly() || Arrays.stream(declaringBean.getConstructorArguments())
+                                    .anyMatch(arg -> declaringBean.getProperty(arg.getName(), arg.getType()).isPresent());
+    }
+
+    /**
+     * This method will attempt to modify the property or if this is a immutable type using a copy constructor to return a new instance with the new value.
+     *
+     * <p>This differs from {@link #set(Object, Object)} which will throw an exception if the property does not have a setter.</p>
+     *
+     * <ul>
+     *     <li>If the property is read-only but can be provided via constructor argument a new instance representing a copy of the bean is returned.</li>
+     *     <li>If the property is mutable then the passed instance is returned and {@link #set(Object, Object)} invoked  to mutate the property</li>
+     *     <li>If there is no way for the property to be mutated then an {@link UnsupportedOperationException} is thrown</li>
+     * </ul>
+     *
+     * @param bean The bean
+     * @param value The new value
+     * @return Either the existing instance or the property is mutable or a newly created instance via the copy constructor pattern.
+     * @throws UnsupportedOperationException if the property cannot be mutated
+     * @since 2.3.0
+     */
+    default B withValue(@NonNull B bean, @Nullable T value) {
+        if (isReadOnly())  {
+            if (value == get(bean)) {
+                return bean;
+            }
+            BeanIntrospection<B> declaringBean = getDeclaringBean();
+            Argument<?>[] constructorArguments = declaringBean.getConstructorArguments();
+            Object[] values = new Object[constructorArguments.length];
+            boolean found = false;
+            for (int i = 0; i < constructorArguments.length; i++) {
+                Argument<?> constructorArgument = constructorArguments[i];
+                String argumentName = constructorArgument.getName();
+                Class<?> argumentType = constructorArgument.getType();
+                BeanProperty<B, ?> prop = declaringBean
+                        .getProperty(argumentName, argumentType).orElse(null);
+                if (prop == null) {
+                    throw new UnsupportedOperationException("Cannot create copy of type [" + declaringBean.getBeanType() + "]. Constructor contains argument [" + argumentName + "] that is not a readable property");
+                } else if (prop == this) {
+                    found = true;
+                    values[i] = value;
+                } else {
+                    values[i] = prop.get(bean);
+                }
+            }
+            if (found) {
+                B newInstance = declaringBean.instantiate(values);
+                Collection<BeanProperty<B, Object>> beanProperties = declaringBean.getBeanProperties();
+                for (BeanProperty<B, Object> beanProperty : beanProperties) {
+                    if (beanProperty != this && beanProperty.isReadWrite()) {
+                        beanProperty.set(newInstance, beanProperty.get(bean));
+                    }
+                }
+                return newInstance;
+            } else {
+                B newInstance = declaringBean.instantiate(values);
+                Collection<BeanProperty<B, Object>> beanProperties = declaringBean.getBeanProperties();
+                for (BeanProperty<B, Object> beanProperty : beanProperties) {
+                    if (beanProperty == this && beanProperty.isReadWrite()) {
+                        found = true;
+                        beanProperty.set(newInstance, beanProperty.get(bean));
+                    }
+                }
+                if (!found) {
+                    throw new UnsupportedOperationException("Cannot mutate property [" + getName() + "] that is not mutable via a setter method or constructor argument for type: " + declaringBean.getBeanType().getName());
+                } else {
+                    return newInstance;
+                }
+            }
+        } else {
+            set(bean, value);
+            return bean;
+        }
+    }
+
     /**
      * Write the bean value.
      * @param bean The bean
      * @param value The value to write
      * @throws IllegalArgumentException If either the bean type or value type are not correct
      */
-    default void set(@Nonnull B bean, @Nullable T value) {
+    default void set(@NonNull B bean, @Nullable T value) {
         if (isReadOnly()) {
             throw new UnsupportedOperationException("Cannot write read-only property: " + getName());
         } else {
@@ -142,7 +229,7 @@ public interface BeanProperty<B, T> extends AnnotatedElement, AnnotationMetadata
      * @param value The value
      * @throws io.micronaut.core.convert.exceptions.ConversionErrorException If the value couldn't be converted
      */
-    default void convertAndSet(@Nonnull B bean, @Nullable Object value) {
+    default void convertAndSet(@NonNull B bean, @Nullable Object value) {
         ArgumentUtils.requireNonNull("bean", bean);
         if (value != null) {
             final Argument<T> argument = asArgument();
@@ -161,13 +248,15 @@ public interface BeanProperty<B, T> extends AnnotatedElement, AnnotationMetadata
     /**
      * @return The property type.
      */
-    @Nonnull Class<T> getType();
+    @NonNull Class<T> getType();
 
     /**
      * Represent the type as an argument, including any generic type information.
      *
      * @return The argument
      */
+    @Override
+    @NonNull
     default Argument<T> asArgument() {
         return Argument.of(getType());
     }
