@@ -5,7 +5,7 @@
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ * https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -15,14 +15,17 @@
  */
 package io.micronaut.inject;
 
+import io.micronaut.core.annotation.NonNull;
+import io.micronaut.core.beans.BeanConstructor;
+
 /**
  * A constructor injection point.
  *
- * @param <T> The constructor type
+ * @param <T> The constructed type
  * @author Graeme Rocher
  * @since 1.0
  */
-public interface ConstructorInjectionPoint<T> extends CallableInjectionPoint {
+public interface ConstructorInjectionPoint<T> extends CallableInjectionPoint<T>, BeanConstructor<T> {
 
     /**
      * Invoke the constructor.
@@ -31,4 +34,14 @@ public interface ConstructorInjectionPoint<T> extends CallableInjectionPoint {
      * @return The new value
      */
     T invoke(Object... args);
+
+    @Override
+    default @NonNull Class<T> getDeclaringBeanType() {
+        return getDeclaringBean().getBeanType();
+    }
+
+    @Override
+    default @NonNull T instantiate(Object... parameterValues) {
+        return invoke(parameterValues);
+    }
 }
