@@ -1,11 +1,11 @@
 /*
- * Copyright 2017-2018 original authors
+ * Copyright 2017-2020 original authors
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ * https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -13,7 +13,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package io.micronaut.spring.beans;
 
 import io.micronaut.context.DefaultApplicationContext;
@@ -21,6 +20,7 @@ import io.micronaut.context.DefaultBeanContext;
 import io.micronaut.context.Qualifier;
 import io.micronaut.context.env.DefaultEnvironment;
 import io.micronaut.core.convert.ArgumentConversionContext;
+import io.micronaut.core.util.ArrayUtils;
 import io.micronaut.inject.qualifiers.Qualifiers;
 import org.springframework.beans.BeansException;
 import org.springframework.beans.factory.DisposableBean;
@@ -31,7 +31,6 @@ import org.springframework.beans.factory.support.DefaultListableBeanFactory;
 import org.springframework.context.EnvironmentAware;
 import org.springframework.core.env.Environment;
 
-import javax.annotation.Nullable;
 import java.lang.annotation.Annotation;
 import java.util.Arrays;
 import java.util.List;
@@ -68,8 +67,9 @@ public class MicronautBeanProcessor implements BeanFactoryPostProcessor, Disposa
     @Override
     public void postProcessBeanFactory(ConfigurableListableBeanFactory beanFactory) throws BeansException {
         if (environment != null) {
-            micronautContext = new DefaultApplicationContext(environment.getActiveProfiles()) {
-                DefaultEnvironment env = new DefaultEnvironment(environment.getActiveProfiles()) {
+            String[] profiles = getProfiles();
+            micronautContext = new DefaultApplicationContext(profiles) {
+                DefaultEnvironment env = new DefaultEnvironment(() -> Arrays.asList(profiles)) {
                     @Override
                     public io.micronaut.context.env.Environment start() {
                         return this;
@@ -81,17 +81,17 @@ public class MicronautBeanProcessor implements BeanFactoryPostProcessor, Disposa
                     }
 
                     @Override
-                    public boolean containsProperty(@Nullable String name) {
+                    public boolean containsProperty(String name) {
                         return environment.containsProperty(name);
                     }
 
                     @Override
-                    public boolean containsProperties(@Nullable String name) {
+                    public boolean containsProperties(String name) {
                         return environment.containsProperty(name);
                     }
 
                     @Override
-                    public <T> Optional<T> getProperty(@Nullable String name, ArgumentConversionContext<T> conversionContext) {
+                    public <T> Optional<T> getProperty(String name, ArgumentConversionContext<T> conversionContext) {
                         return Optional.ofNullable(environment.getProperty(name, conversionContext.getArgument().getType()));
                     }
                 };
@@ -107,16 +107,19 @@ public class MicronautBeanProcessor implements BeanFactoryPostProcessor, Disposa
         micronautContext.start();
 
         micronautBeanQualifierTypes
-                .stream()
                 .forEach(micronautBeanQualifierType -> {
             Qualifier<Object> micronautBeanQualifier;
             if (micronautBeanQualifierType.isAnnotation()) {
-                micronautBeanQualifier = Qualifiers.byStereotype((Class<? extends Annotation>) micronautBeanQualifierType);
+                if (micronautBeanQualifierType.getPackage().getName().equals("jakarta.inject")) {
+                    String injectName = micronautBeanQualifierType.getName().replace("jakarta", "javax");
+                    micronautBeanQualifier = Qualifiers.byStereotype(injectName);
+                } else {
+                    micronautBeanQualifier = Qualifiers.byStereotype((Class<? extends Annotation>) micronautBeanQualifierType);
+                }
             } else {
                 micronautBeanQualifier = Qualifiers.byType(micronautBeanQualifierType);
             }
             micronautContext.getBeanDefinitions(micronautBeanQualifier)
-                    .stream()
                     .forEach(definition -> {
                         final BeanDefinitionBuilder beanDefinitionBuilder = BeanDefinitionBuilder
                                 .rootBeanDefinition(MicronautSpringBeanFactory.class.getName());
@@ -128,6 +131,14 @@ public class MicronautBeanProcessor implements BeanFactoryPostProcessor, Disposa
         });
     }
 
+    private String[] getProfiles() {
+        if (ArrayUtils.isNotEmpty(environment.getActiveProfiles())) {
+            return environment.getActiveProfiles();
+        } else {
+            return environment.getDefaultProfiles();
+        }
+    }
+
     @Override
     public void destroy() throws Exception {
         if (micronautContext != null) {
