@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2019 the original author or authors.
+ * Copyright 2012-2020 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -42,7 +42,7 @@ class ProjectGenerator {
 		this.initializrService = initializrService;
 	}
 
-	public void generateProject(ProjectGenerationRequest request, boolean force) throws IOException {
+	void generateProject(ProjectGenerationRequest request, boolean force) throws IOException {
 		ProjectGenerationResponse response = this.initializrService.generate(request);
 		String fileName = (request.getOutput() != null) ? request.getOutput() : response.getFileName();
 		if (shouldExtract(request, response)) {
@@ -58,7 +58,7 @@ class ProjectGenerator {
 		}
 		if (fileName == null) {
 			throw new ReportableException("Could not save the project, the server did not set a preferred "
-					+ "file name and no location was set. Specify the output location " + "for the project.");
+					+ "file name and no location was set. Specify the output location for the project.");
 		}
 		writeProject(response, fileName, force);
 	}
@@ -74,10 +74,7 @@ class ProjectGenerator {
 			return true;
 		}
 		// explicit name hasn't been provided for an archive and there is no extension
-		if (isZipArchive(response) && request.getOutput() != null && !request.getOutput().contains(".")) {
-			return true;
-		}
-		return false;
+		return isZipArchive(response) && request.getOutput() != null && !request.getOutput().contains(".");
 	}
 
 	private boolean isZipArchive(ProjectGenerationResponse entity) {
@@ -93,23 +90,24 @@ class ProjectGenerator {
 	}
 
 	private void extractProject(ProjectGenerationResponse entity, String output, boolean overwrite) throws IOException {
-		File outputFolder = (output != null) ? new File(output) : new File(System.getProperty("user.dir"));
-		if (!outputFolder.exists()) {
-			outputFolder.mkdirs();
+		File outputDirectory = (output != null) ? new File(output) : new File(System.getProperty("user.dir"));
+		if (!outputDirectory.exists()) {
+			outputDirectory.mkdirs();
 		}
 		try (ZipInputStream zipStream = new ZipInputStream(new ByteArrayInputStream(entity.getContent()))) {
-			extractFromStream(zipStream, overwrite, outputFolder);
-			fixExecutableFlag(outputFolder, "mvnw");
-			fixExecutableFlag(outputFolder, "gradlew");
-			Log.info("Project extracted to '" + outputFolder.getAbsolutePath() + "'");
+			extractFromStream(zipStream, overwrite, outputDirectory);
+			fixExecutableFlag(outputDirectory, "mvnw");
+			fixExecutableFlag(outputDirectory, "gradlew");
+			Log.info("Project extracted to '" + outputDirectory.getAbsolutePath() + "'");
 		}
 	}
 
-	private void extractFromStream(ZipInputStream zipStream, boolean overwrite, File outputFolder) throws IOException {
+	private void extractFromStream(ZipInputStream zipStream, boolean overwrite, File outputDirectory)
+			throws IOException {
 		ZipEntry entry = zipStream.getNextEntry();
-		String canonicalOutputPath = outputFolder.getCanonicalPath() + File.separator;
+		String canonicalOutputPath = outputDirectory.getCanonicalPath() + File.separator;
 		while (entry != null) {
-			File file = new File(outputFolder, entry.getName());
+			File file = new File(outputDirectory, entry.getName());
 			String canonicalEntryPath = file.getCanonicalPath();
 			if (!canonicalEntryPath.startsWith(canonicalOutputPath)) {
 				throw new ReportableException("Entry '" + entry.getName() + "' would be written to '"
