@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2018 the original author or authors.
+ * Copyright 2012-2019 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -40,17 +40,16 @@ import org.springframework.boot.cli.compiler.dependencies.DependencyManagementAr
  */
 public class DependencyResolutionContext {
 
-	private final Map<String, Dependency> managedDependencyByGroupAndArtifact = new HashMap<String, Dependency>();
+	private final Map<String, Dependency> managedDependencyByGroupAndArtifact = new HashMap<>();
 
-	private final List<Dependency> managedDependencies = new ArrayList<Dependency>();
+	private final List<Dependency> managedDependencies = new ArrayList<>();
 
 	private DependencyManagement dependencyManagement = null;
 
 	private ArtifactCoordinatesResolver artifactCoordinatesResolver;
 
 	private String getIdentifier(Dependency dependency) {
-		return getIdentifier(dependency.getArtifact().getGroupId(),
-				dependency.getArtifact().getArtifactId());
+		return getIdentifier(dependency.getArtifact().getGroupId(), dependency.getArtifact().getArtifactId());
 	}
 
 	private String getIdentifier(String groupId, String artifactId) {
@@ -64,8 +63,7 @@ public class DependencyResolutionContext {
 	public String getManagedVersion(String groupId, String artifactId) {
 		Dependency dependency = getManagedDependency(groupId, artifactId);
 		if (dependency == null) {
-			dependency = this.managedDependencyByGroupAndArtifact
-					.get(getIdentifier(groupId, artifactId));
+			dependency = this.managedDependencyByGroupAndArtifact.get(getIdentifier(groupId, artifactId));
 		}
 		return (dependency != null) ? dependency.getArtifact().getVersion() : null;
 	}
@@ -75,38 +73,32 @@ public class DependencyResolutionContext {
 	}
 
 	private Dependency getManagedDependency(String group, String artifact) {
-		return this.managedDependencyByGroupAndArtifact
-				.get(getIdentifier(group, artifact));
+		return this.managedDependencyByGroupAndArtifact.get(getIdentifier(group, artifact));
 	}
 
 	public void addManagedDependencies(List<Dependency> dependencies) {
 		this.managedDependencies.addAll(dependencies);
 		for (Dependency dependency : dependencies) {
-			this.managedDependencyByGroupAndArtifact.put(getIdentifier(dependency),
-					dependency);
+			this.managedDependencyByGroupAndArtifact.put(getIdentifier(dependency), dependency);
 		}
 	}
 
 	public void addDependencyManagement(DependencyManagement dependencyManagement) {
 		for (org.springframework.boot.cli.compiler.dependencies.Dependency dependency : dependencyManagement
 				.getDependencies()) {
-			List<Exclusion> aetherExclusions = new ArrayList<Exclusion>();
+			List<Exclusion> aetherExclusions = new ArrayList<>();
 			for (org.springframework.boot.cli.compiler.dependencies.Dependency.Exclusion exclusion : dependency
 					.getExclusions()) {
-				aetherExclusions.add(new Exclusion(exclusion.getGroupId(),
-						exclusion.getArtifactId(), "*", "*"));
+				aetherExclusions.add(new Exclusion(exclusion.getGroupId(), exclusion.getArtifactId(), "*", "*"));
 			}
-			Dependency aetherDependency = new Dependency(
-					new DefaultArtifact(dependency.getGroupId(),
-							dependency.getArtifactId(), "jar", dependency.getVersion()),
-					JavaScopes.COMPILE, false, aetherExclusions);
+			Dependency aetherDependency = new Dependency(new DefaultArtifact(dependency.getGroupId(),
+					dependency.getArtifactId(), "jar", dependency.getVersion()), JavaScopes.COMPILE, false,
+					aetherExclusions);
 			this.managedDependencies.add(0, aetherDependency);
-			this.managedDependencyByGroupAndArtifact.put(getIdentifier(aetherDependency),
-					aetherDependency);
+			this.managedDependencyByGroupAndArtifact.put(getIdentifier(aetherDependency), aetherDependency);
 		}
 		this.dependencyManagement = (this.dependencyManagement != null)
-				? new CompositeDependencyManagement(dependencyManagement,
-						this.dependencyManagement)
+				? new CompositeDependencyManagement(dependencyManagement, this.dependencyManagement)
 				: dependencyManagement;
 		this.artifactCoordinatesResolver = new DependencyManagementArtifactCoordinatesResolver(
 				this.dependencyManagement);
