@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2016 the original author or authors.
+ * Copyright 2012-2019 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -34,8 +34,7 @@ import com.fasterxml.jackson.databind.SerializerProvider;
 public abstract class JsonObjectSerializer<T> extends JsonSerializer<T> {
 
 	@Override
-	public final void serialize(T value, JsonGenerator jgen, SerializerProvider provider)
-			throws IOException {
+	public final void serialize(T value, JsonGenerator jgen, SerializerProvider provider) throws IOException {
 		try {
 			jgen.writeStartObject();
 			serializeObject(value, jgen, provider);
@@ -56,7 +55,7 @@ public abstract class JsonObjectSerializer<T> extends JsonSerializer<T> {
 	 * @param provider the serializer provider
 	 * @throws IOException on error
 	 */
-	protected abstract void serializeObject(T value, JsonGenerator jgen,
-			SerializerProvider provider) throws IOException;
+	protected abstract void serializeObject(T value, JsonGenerator jgen, SerializerProvider provider)
+			throws IOException;
 
 }
