@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2019 the original author or authors.
+ * Copyright 2012-2021 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,8 +17,11 @@
 package org.springframework.boot.jdbc;
 
 import java.sql.Connection;
+import java.sql.DatabaseMetaData;
 import java.sql.SQLException;
 import java.util.Locale;
+import java.util.function.Predicate;
+import java.util.stream.Stream;
 
 import javax.sql.DataSource;
 
@@ -35,6 +38,7 @@ import org.springframework.util.ClassUtils;
  * @author Phillip Webb
  * @author Dave Syer
  * @author Stephane Nicoll
+ * @author Nidhi Desai
  * @since 1.0.0
  * @see #get(ClassLoader)
  */
@@ -43,33 +47,49 @@ public enum EmbeddedDatabaseConnection {
 	/**
 	 * No Connection.
 	 */
-	NONE(null, null, null),
+	NONE(null, null, null, (url) -> false),
 
 	/**
 	 * H2 Database Connection.
 	 */
-	H2(EmbeddedDatabaseType.H2, "org.h2.Driver", "jdbc:h2:mem:%s;DB_CLOSE_DELAY=-1;DB_CLOSE_ON_EXIT=FALSE"),
+	H2(EmbeddedDatabaseType.H2, DatabaseDriver.H2.getDriverClassName(),
+			"jdbc:h2:mem:%s;DB_CLOSE_DELAY=-1;DB_CLOSE_ON_EXIT=FALSE", (url) -> url.contains(":h2:mem")),
 
 	/**
 	 * Derby Database Connection.
 	 */
-	DERBY(EmbeddedDatabaseType.DERBY, "org.apache.derby.jdbc.EmbeddedDriver", "jdbc:derby:memory:%s;create=true"),
+	DERBY(EmbeddedDatabaseType.DERBY, DatabaseDriver.DERBY.getDriverClassName(), "jdbc:derby:memory:%s;create=true",
+			(url) -> true),
 
 	/**
 	 * HSQL Database Connection.
+	 * @since 2.4.0
 	 */
-	HSQL(EmbeddedDatabaseType.HSQL, "org.hsqldb.jdbcDriver", "jdbc:hsqldb:mem:%s");
+	HSQLDB(EmbeddedDatabaseType.HSQL, DatabaseDriver.HSQLDB.getDriverClassName(), "org.hsqldb.jdbcDriver",
+			"jdbc:hsqldb:mem:%s", (url) -> url.contains(":hsqldb:mem:"));
 
 	private final EmbeddedDatabaseType type;
 
 	private final String driverClass;
 
+	private final String alternativeDriverClass;
+
 	private final String url;
 
-	EmbeddedDatabaseConnection(EmbeddedDatabaseType type, String driverClass, String url) {
+	private final Predicate<String> embeddedUrl;
+
+	EmbeddedDatabaseConnection(EmbeddedDatabaseType type, String driverClass, String url,
+			Predicate<String> embeddedUrl) {
+		this(type, driverClass, null, url, embeddedUrl);
+	}
+
+	EmbeddedDatabaseConnection(EmbeddedDatabaseType type, String driverClass, String fallbackDriverClass, String url,
+			Predicate<String> embeddedUrl) {
 		this.type = type;
 		this.driverClass = driverClass;
+		this.alternativeDriverClass = fallbackDriverClass;
 		this.url = url;
+		this.embeddedUrl = embeddedUrl;
 	}
 
 	/**
@@ -98,15 +118,37 @@ public enum EmbeddedDatabaseConnection {
 		return (this.url != null) ? String.format(this.url, databaseName) : null;
 	}
 
+	boolean isEmbeddedUrl(String url) {
+		return this.embeddedUrl.test(url);
+	}
+
+	boolean isDriverCompatible(String driverClass) {
+		return (driverClass != null
+				&& (driverClass.equals(this.driverClass) || driverClass.equals(this.alternativeDriverClass)));
+	}
+
 	/**
-	 * Convenience method to determine if a given driver class name represents an embedded
-	 * database type.
+	 * Convenience method to determine if a given driver class name and url represent an
+	 * embedded database type.
 	 * @param driverClass the driver class
-	 * @return true if the driver class is one of the embedded types
+	 * @param url the jdbc url (can be {@code null})
+	 * @return true if the driver class and url refer to an embedded database
+	 * @since 2.4.0
 	 */
-	public static boolean isEmbedded(String driverClass) {
-		return driverClass != null && (driverClass.equals(HSQL.driverClass) || driverClass.equals(H2.driverClass)
-				|| driverClass.equals(DERBY.driverClass));
+	public static boolean isEmbedded(String driverClass, String url) {
+		if (driverClass == null) {
+			return false;
+		}
+		EmbeddedDatabaseConnection connection = getEmbeddedDatabaseConnection(driverClass);
+		if (connection == NONE) {
+			return false;
+		}
+		return (url == null || connection.isEmbeddedUrl(url));
+	}
+
+	private static EmbeddedDatabaseConnection getEmbeddedDatabaseConnection(String driverClass) {
+		return Stream.of(H2, HSQLDB, DERBY).filter((connection) -> connection.isDriverCompatible(driverClass))
+				.findFirst().orElse(NONE);
 	}
 
 	/**
@@ -147,15 +189,17 @@ public enum EmbeddedDatabaseConnection {
 
 		@Override
 		public Boolean doInConnection(Connection connection) throws SQLException, DataAccessException {
-			String productName = connection.getMetaData().getDatabaseProductName();
+			DatabaseMetaData metaData = connection.getMetaData();
+			String productName = metaData.getDatabaseProductName();
 			if (productName == null) {
 				return false;
 			}
 			productName = productName.toUpperCase(Locale.ENGLISH);
 			EmbeddedDatabaseConnection[] candidates = EmbeddedDatabaseConnection.values();
 			for (EmbeddedDatabaseConnection candidate : candidates) {
-				if (candidate != NONE && productName.contains(candidate.name())) {
-					return true;
+				if (candidate != NONE && productName.contains(candidate.getType().name())) {
+					String url = metaData.getURL();
+					return (url == null || candidate.isEmbeddedUrl(url));
 				}
 			}
 			return false;
