@@ -45,16 +45,16 @@ class LogbackConfigurator {
 		this.context = context;
 	}
 
-	public LoggerContext getContext() {
+	LoggerContext getContext() {
 		return this.context;
 	}
 
-	public Object getConfigurationLock() {
+	Object getConfigurationLock() {
 		return this.context.getConfigurationLock();
 	}
 
 	@SuppressWarnings({ "rawtypes", "unchecked" })
-	public void conversionRule(String conversionWord, Class<? extends Converter> converterClass) {
+	void conversionRule(String conversionWord, Class<? extends Converter> converterClass) {
 		Assert.hasLength(conversionWord, "Conversion word must not be empty");
 		Assert.notNull(converterClass, "Converter class must not be null");
 		Map<String, String> registry = (Map<String, String>) this.context
@@ -66,20 +66,20 @@ class LogbackConfigurator {
 		registry.put(conversionWord, converterClass.getName());
 	}
 
-	public void appender(String name, Appender<?> appender) {
+	void appender(String name, Appender<?> appender) {
 		appender.setName(name);
 		start(appender);
 	}
 
-	public void logger(String name, Level level) {
+	void logger(String name, Level level) {
 		logger(name, level, true);
 	}
 
-	public void logger(String name, Level level, boolean additive) {
+	void logger(String name, Level level, boolean additive) {
 		logger(name, level, additive, null);
 	}
 
-	public void logger(String name, Level level, boolean additive, Appender<ILoggingEvent> appender) {
+	void logger(String name, Level level, boolean additive, Appender<ILoggingEvent> appender) {
 		Logger logger = this.context.getLogger(name);
 		if (level != null) {
 			logger.setLevel(level);
@@ -91,7 +91,7 @@ class LogbackConfigurator {
 	}
 
 	@SafeVarargs
-	public final void root(Level level, Appender<ILoggingEvent>... appenders) {
+	final void root(Level level, Appender<ILoggingEvent>... appenders) {
 		Logger logger = this.context.getLogger(org.slf4j.Logger.ROOT_LOGGER_NAME);
 		if (level != null) {
 			logger.setLevel(level);
@@ -101,7 +101,7 @@ class LogbackConfigurator {
 		}
 	}
 
-	public void start(LifeCycle lifeCycle) {
+	void start(LifeCycle lifeCycle) {
 		if (lifeCycle instanceof ContextAware) {
 			((ContextAware) lifeCycle).setContext(this.context);
 		}
