@@ -1,11 +1,11 @@
 /*
- * Copyright 2002-2016 the original author or authors.
+ * Copyright 2002-2022 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -17,20 +17,28 @@
 package org.springframework.context.support;
 
 import java.io.IOException;
+import java.lang.reflect.Constructor;
 import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.function.Supplier;
 
+import org.springframework.beans.BeanUtils;
 import org.springframework.beans.BeansException;
 import org.springframework.beans.factory.BeanDefinitionStoreException;
 import org.springframework.beans.factory.NoSuchBeanDefinitionException;
 import org.springframework.beans.factory.config.AutowireCapableBeanFactory;
 import org.springframework.beans.factory.config.BeanDefinition;
+import org.springframework.beans.factory.config.BeanDefinitionCustomizer;
 import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
 import org.springframework.beans.factory.support.BeanDefinitionRegistry;
 import org.springframework.beans.factory.support.DefaultListableBeanFactory;
+import org.springframework.beans.factory.support.RootBeanDefinition;
 import org.springframework.context.ApplicationContext;
+import org.springframework.core.io.ProtocolResolver;
 import org.springframework.core.io.Resource;
 import org.springframework.core.io.ResourceLoader;
 import org.springframework.core.io.support.ResourcePatternResolver;
+import org.springframework.core.metrics.ApplicationStartup;
+import org.springframework.lang.Nullable;
 import org.springframework.util.Assert;
 
 /**
@@ -79,6 +87,7 @@ import org.springframework.util.Assert;
  *
  * @author Juergen Hoeller
  * @author Chris Beams
+ * @author Sam Brannen
  * @since 1.1.2
  * @see #registerBeanDefinition
  * @see #refresh()
@@ -89,6 +98,7 @@ public class GenericApplicationContext extends AbstractApplicationContext implem
 
 	private final DefaultListableBeanFactory beanFactory;
 
+	@Nullable
 	private ResourceLoader resourceLoader;
 
 	private boolean customClassLoader = false;
@@ -122,7 +132,7 @@ public class GenericApplicationContext extends AbstractApplicationContext implem
 	 * @see #registerBeanDefinition
 	 * @see #refresh
 	 */
-	public GenericApplicationContext(ApplicationContext parent) {
+	public GenericApplicationContext(@Nullable ApplicationContext parent) {
 		this();
 		setParent(parent);
 	}
@@ -146,14 +156,15 @@ public class GenericApplicationContext extends AbstractApplicationContext implem
 	 * @see org.springframework.beans.factory.config.ConfigurableBeanFactory#setParentBeanFactory
 	 */
 	@Override
-	public void setParent(ApplicationContext parent) {
+	public void setParent(@Nullable ApplicationContext parent) {
 		super.setParent(parent);
 		this.beanFactory.setParentBeanFactory(getInternalParentBeanFactory());
 	}
 
 	@Override
-	public void setId(String id) {
-		super.setId(id);
+	public void setApplicationStartup(ApplicationStartup applicationStartup) {
+		super.setApplicationStartup(applicationStartup);
+		this.beanFactory.setApplicationStartup(applicationStartup);
 	}
 
 	/**
@@ -207,13 +218,23 @@ public class GenericApplicationContext extends AbstractApplicationContext implem
 	//---------------------------------------------------------------------
 
 	/**
-	 * This implementation delegates to this context's ResourceLoader if set,
-	 * falling back to the default superclass behavior else.
-	 * @see #setResourceLoader
+	 * This implementation delegates to this context's {@code ResourceLoader} if set,
+	 * falling back to the default superclass behavior otherwise.
+	 * <p>As of Spring Framework 5.3.22, this method also honors registered
+	 * {@linkplain #getProtocolResolvers() protocol resolvers} when a custom
+	 * {@code ResourceLoader} has been set.
+	 * @see #setResourceLoader(ResourceLoader)
+	 * @see #addProtocolResolver(ProtocolResolver)
 	 */
 	@Override
 	public Resource getResource(String location) {
 		if (this.resourceLoader != null) {
+			for (ProtocolResolver protocolResolver : getProtocolResolvers()) {
+				Resource resource = protocolResolver.resolve(location, this);
+				if (resource != null) {
+					return resource;
+				}
+			}
 			return this.resourceLoader.getResource(location);
 		}
 		return super.getResource(location);
@@ -222,7 +243,7 @@ public class GenericApplicationContext extends AbstractApplicationContext implem
 	/**
 	 * This implementation delegates to this context's ResourceLoader if it
 	 * implements the ResourcePatternResolver interface, falling back to the
-	 * default superclass behavior else.
+	 * default superclass behavior otherwise.
 	 * @see #setResourceLoader
 	 */
 	@Override
@@ -234,12 +255,13 @@ public class GenericApplicationContext extends AbstractApplicationContext implem
 	}
 
 	@Override
-	public void setClassLoader(ClassLoader classLoader) {
+	public void setClassLoader(@Nullable ClassLoader classLoader) {
 		super.setClassLoader(classLoader);
 		this.customClassLoader = true;
 	}
 
 	@Override
+	@Nullable
 	public ClassLoader getClassLoader() {
 		if (this.resourceLoader != null && !this.customClassLoader) {
 			return this.resourceLoader.getClassLoader();
@@ -350,4 +372,159 @@ public class GenericApplicationContext extends AbstractApplicationContext implem
 		return this.beanFactory.isAlias(beanName);
 	}
 
+
+	//---------------------------------------------------------------------
+	// Convenient methods for registering individual beans
+	//---------------------------------------------------------------------
+
+	/**
+	 * Register a bean from the given bean class, optionally providing explicit
+	 * constructor arguments for consideration in the autowiring process.
+	 * @param beanClass the class of the bean
+	 * @param constructorArgs custom argument values to be fed into Spring's
+	 * constructor resolution algorithm, resolving either all arguments or just
+	 * specific ones, with the rest to be resolved through regular autowiring
+	 * (may be {@code null} or empty)
+	 * @since 5.2 (since 5.0 on the AnnotationConfigApplicationContext subclass)
+	 */
+	public <T> void registerBean(Class<T> beanClass, Object... constructorArgs) {
+		registerBean(null, beanClass, constructorArgs);
+	}
+
+	/**
+	 * Register a bean from the given bean class, optionally providing explicit
+	 * constructor arguments for consideration in the autowiring process.
+	 * @param beanName the name of the bean (may be {@code null})
+	 * @param beanClass the class of the bean
+	 * @param constructorArgs custom argument values to be fed into Spring's
+	 * constructor resolution algorithm, resolving either all arguments or just
+	 * specific ones, with the rest to be resolved through regular autowiring
+	 * (may be {@code null} or empty)
+	 * @since 5.2 (since 5.0 on the AnnotationConfigApplicationContext subclass)
+	 */
+	public <T> void registerBean(@Nullable String beanName, Class<T> beanClass, Object... constructorArgs) {
+		registerBean(beanName, beanClass, (Supplier<T>) null,
+				bd -> {
+					for (Object arg : constructorArgs) {
+						bd.getConstructorArgumentValues().addGenericArgumentValue(arg);
+					}
+				});
+	}
+
+	/**
+	 * Register a bean from the given bean class, optionally customizing its
+	 * bean definition metadata (typically declared as a lambda expression).
+	 * @param beanClass the class of the bean (resolving a public constructor
+	 * to be autowired, possibly simply the default constructor)
+	 * @param customizers one or more callbacks for customizing the factory's
+	 * {@link BeanDefinition}, e.g. setting a lazy-init or primary flag
+	 * @since 5.0
+	 * @see #registerBean(String, Class, Supplier, BeanDefinitionCustomizer...)
+	 */
+	public final <T> void registerBean(Class<T> beanClass, BeanDefinitionCustomizer... customizers) {
+		registerBean(null, beanClass, null, customizers);
+	}
+
+	/**
+	 * Register a bean from the given bean class, optionally customizing its
+	 * bean definition metadata (typically declared as a lambda expression).
+	 * @param beanName the name of the bean (may be {@code null})
+	 * @param beanClass the class of the bean (resolving a public constructor
+	 * to be autowired, possibly simply the default constructor)
+	 * @param customizers one or more callbacks for customizing the factory's
+	 * {@link BeanDefinition}, e.g. setting a lazy-init or primary flag
+	 * @since 5.0
+	 * @see #registerBean(String, Class, Supplier, BeanDefinitionCustomizer...)
+	 */
+	public final <T> void registerBean(
+			@Nullable String beanName, Class<T> beanClass, BeanDefinitionCustomizer... customizers) {
+
+		registerBean(beanName, beanClass, null, customizers);
+	}
+
+	/**
+	 * Register a bean from the given bean class, using the given supplier for
+	 * obtaining a new instance (typically declared as a lambda expression or
+	 * method reference), optionally customizing its bean definition metadata
+	 * (again typically declared as a lambda expression).
+	 * @param beanClass the class of the bean
+	 * @param supplier a callback for creating an instance of the bean
+	 * @param customizers one or more callbacks for customizing the factory's
+	 * {@link BeanDefinition}, e.g. setting a lazy-init or primary flag
+	 * @since 5.0
+	 * @see #registerBean(String, Class, Supplier, BeanDefinitionCustomizer...)
+	 */
+	public final <T> void registerBean(
+			Class<T> beanClass, Supplier<T> supplier, BeanDefinitionCustomizer... customizers) {
+
+		registerBean(null, beanClass, supplier, customizers);
+	}
+
+	/**
+	 * Register a bean from the given bean class, using the given supplier for
+	 * obtaining a new instance (typically declared as a lambda expression or
+	 * method reference), optionally customizing its bean definition metadata
+	 * (again typically declared as a lambda expression).
+	 * <p>This method can be overridden to adapt the registration mechanism for
+	 * all {@code registerBean} methods (since they all delegate to this one).
+	 * @param beanName the name of the bean (may be {@code null})
+	 * @param beanClass the class of the bean
+	 * @param supplier a callback for creating an instance of the bean (in case
+	 * of {@code null}, resolving a public constructor to be autowired instead)
+	 * @param customizers one or more callbacks for customizing the factory's
+	 * {@link BeanDefinition}, e.g. setting a lazy-init or primary flag
+	 * @since 5.0
+	 */
+	public <T> void registerBean(@Nullable String beanName, Class<T> beanClass,
+			@Nullable Supplier<T> supplier, BeanDefinitionCustomizer... customizers) {
+
+		ClassDerivedBeanDefinition beanDefinition = new ClassDerivedBeanDefinition(beanClass);
+		if (supplier != null) {
+			beanDefinition.setInstanceSupplier(supplier);
+		}
+		for (BeanDefinitionCustomizer customizer : customizers) {
+			customizer.customize(beanDefinition);
+		}
+
+		String nameToUse = (beanName != null ? beanName : beanClass.getName());
+		registerBeanDefinition(nameToUse, beanDefinition);
+	}
+
+
+	/**
+	 * {@link RootBeanDefinition} marker subclass for {@code #registerBean} based
+	 * registrations with flexible autowiring for public constructors.
+	 */
+	@SuppressWarnings("serial")
+	private static class ClassDerivedBeanDefinition extends RootBeanDefinition {
+
+		public ClassDerivedBeanDefinition(Class<?> beanClass) {
+			super(beanClass);
+		}
+
+		public ClassDerivedBeanDefinition(ClassDerivedBeanDefinition original) {
+			super(original);
+		}
+
+		@Override
+		@Nullable
+		public Constructor<?>[] getPreferredConstructors() {
+			Class<?> clazz = getBeanClass();
+			Constructor<?> primaryCtor = BeanUtils.findPrimaryConstructor(clazz);
+			if (primaryCtor != null) {
+				return new Constructor<?>[] {primaryCtor};
+			}
+			Constructor<?>[] publicCtors = clazz.getConstructors();
+			if (publicCtors.length > 0) {
+				return publicCtors;
+			}
+			return null;
+		}
+
+		@Override
+		public RootBeanDefinition cloneBeanDefinition() {
+			return new ClassDerivedBeanDefinition(this);
+		}
+	}
+
 }
