@@ -1,11 +1,11 @@
 /*
- * Copyright 2002-2016 the original author or authors.
+ * Copyright 2002-2020 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -16,17 +16,12 @@
 
 package org.springframework.core;
 
-import java.io.Externalizable;
-import java.io.Serializable;
 import java.lang.reflect.Method;
 import java.lang.reflect.Proxy;
-import java.util.Arrays;
 import java.util.Collection;
-import java.util.Collections;
-import java.util.HashSet;
 import java.util.Iterator;
-import java.util.Set;
 
+import org.springframework.lang.Nullable;
 import org.springframework.util.Assert;
 import org.springframework.util.ClassUtils;
 
@@ -36,39 +31,33 @@ import org.springframework.util.ClassUtils;
  *
  * @author Rob Harrop
  * @author Juergen Hoeller
+ * @author Rossen Stoyanchev
  * @since 2.0
  */
-public abstract class Conventions {
+public final class Conventions {
 
 	/**
 	 * Suffix added to names when using arrays.
 	 */
 	private static final String PLURAL_SUFFIX = "List";
 
-	/**
-	 * Set of interfaces that are supposed to be ignored
-	 * when searching for the 'primary' interface of a proxy.
-	 */
-	private static final Set<Class<?>> IGNORED_INTERFACES;
 
-	static {
-		IGNORED_INTERFACES = Collections.unmodifiableSet(new HashSet<Class<?>>(Arrays.<Class<?>>asList(
-				Serializable.class, Externalizable.class, Cloneable.class, Comparable.class)));
+	private Conventions() {
 	}
 
 
 	/**
-	 * Determine the conventional variable name for the supplied
-	 * {@code Object} based on its concrete type. The convention
-	 * used is to return the uncapitalized short name of the {@code Class},
-	 * according to JavaBeans property naming rules: So,
-	 * {@code com.myapp.Product} becomes {@code product};
-	 * {@code com.myapp.MyProduct} becomes {@code myProduct};
-	 * {@code com.myapp.UKProduct} becomes {@code UKProduct}.
-	 * <p>For arrays, we use the pluralized version of the array component type.
-	 * For {@code Collection}s we attempt to 'peek ahead' in the
-	 * {@code Collection} to determine the component type and
-	 * return the pluralized version of that component type.
+	 * Determine the conventional variable name for the supplied {@code Object}
+	 * based on its concrete type. The convention used is to return the
+	 * un-capitalized short name of the {@code Class}, according to JavaBeans
+	 * property naming rules.
+	 * <p>For example:<br>
+	 * {@code com.myapp.Product} becomes {@code "product"}<br>
+	 * {@code com.myapp.MyProduct} becomes {@code "myProduct"}<br>
+	 * {@code com.myapp.UKProduct} becomes {@code "UKProduct"}<br>
+	 * <p>For arrays the pluralized version of the array component type is used.
+	 * For {@code Collection}s an attempt is made to 'peek ahead' to determine
+	 * the component type and return its pluralized version.
 	 * @param value the value to generate a variable name for
 	 * @return the generated variable name
 	 */
@@ -84,7 +73,8 @@ public abstract class Conventions {
 		else if (value instanceof Collection) {
 			Collection<?> collection = (Collection<?>) value;
 			if (collection.isEmpty()) {
-				throw new IllegalArgumentException("Cannot generate variable name for an empty Collection");
+				throw new IllegalArgumentException(
+						"Cannot generate variable name for an empty Collection");
 			}
 			Object valueToCheck = peekAhead(collection);
 			valueClass = getClassForValue(valueToCheck);
@@ -99,22 +89,27 @@ public abstract class Conventions {
 	}
 
 	/**
-	 * Determine the conventional variable name for the supplied parameter,
-	 * taking the generic collection type (if any) into account.
-	 * @param parameter the method or constructor parameter to generate a variable name for
+	 * Determine the conventional variable name for the given parameter taking
+	 * the generic collection type, if any, into account.
+	 * <p>As of 5.0 this method supports reactive types:<br>
+	 * {@code Mono<com.myapp.Product>} becomes {@code "productMono"}<br>
+	 * {@code Flux<com.myapp.MyProduct>} becomes {@code "myProductFlux"}<br>
+	 * {@code Observable<com.myapp.MyProduct>} becomes {@code "myProductObservable"}<br>
+	 * @param parameter the method or constructor parameter
 	 * @return the generated variable name
 	 */
 	public static String getVariableNameForParameter(MethodParameter parameter) {
 		Assert.notNull(parameter, "MethodParameter must not be null");
 		Class<?> valueClass;
 		boolean pluralize = false;
+		String reactiveSuffix = "";
 
 		if (parameter.getParameterType().isArray()) {
 			valueClass = parameter.getParameterType().getComponentType();
 			pluralize = true;
 		}
 		else if (Collection.class.isAssignableFrom(parameter.getParameterType())) {
-			valueClass = GenericCollectionTypeResolver.getCollectionParameterType(parameter);
+			valueClass = ResolvableType.forMethodParameter(parameter).asCollection().resolveGeneric();
 			if (valueClass == null) {
 				throw new IllegalArgumentException(
 						"Cannot generate variable name for non-typed Collection parameter type");
@@ -123,15 +118,20 @@ public abstract class Conventions {
 		}
 		else {
 			valueClass = parameter.getParameterType();
+			ReactiveAdapter adapter = ReactiveAdapterRegistry.getSharedInstance().getAdapter(valueClass);
+			if (adapter != null && !adapter.getDescriptor().isNoValue()) {
+				reactiveSuffix = ClassUtils.getShortName(valueClass);
+				valueClass = parameter.nested().getNestedParameterType();
+			}
 		}
 
 		String name = ClassUtils.getShortNameAsProperty(valueClass);
-		return (pluralize ? pluralize(name) : name);
+		return (pluralize ? pluralize(name) : name + reactiveSuffix);
 	}
 
 	/**
-	 * Determine the conventional variable name for the return type of the supplied method,
-	 * taking the generic collection type (if any) into account.
+	 * Determine the conventional variable name for the return type of the
+	 * given method, taking the generic collection type, if any, into account.
 	 * @param method the method to generate a variable name for
 	 * @return the generated variable name
 	 */
@@ -140,56 +140,62 @@ public abstract class Conventions {
 	}
 
 	/**
-	 * Determine the conventional variable name for the return type of the supplied method,
-	 * taking the generic collection type (if any) into account, falling back to the
-	 * given return value if the method declaration is not specific enough (i.e. in case of
-	 * the return type being declared as {@code Object} or as untyped collection).
+	 * Determine the conventional variable name for the return type of the given
+	 * method, taking the generic collection type, if any, into account, falling
+	 * back on the given actual return value if the method declaration is not
+	 * specific enough, e.g. {@code Object} return type or untyped collection.
 	 * @param method the method to generate a variable name for
 	 * @param value the return value (may be {@code null} if not available)
 	 * @return the generated variable name
 	 */
-	public static String getVariableNameForReturnType(Method method, Object value) {
+	public static String getVariableNameForReturnType(Method method, @Nullable Object value) {
 		return getVariableNameForReturnType(method, method.getReturnType(), value);
 	}
 
 	/**
-	 * Determine the conventional variable name for the return type of the supplied method,
-	 * taking the generic collection type (if any) into account, falling back to the
-	 * given return value if the method declaration is not specific enough (i.e. in case of
-	 * the return type being declared as {@code Object} or as untyped collection).
+	 * Determine the conventional variable name for the return type of the given
+	 * method, taking the generic collection type, if any, into account, falling
+	 * back on the given return value if the method declaration is not specific
+	 * enough, e.g. {@code Object} return type or untyped collection.
+	 * <p>As of 5.0 this method supports reactive types:<br>
+	 * {@code Mono<com.myapp.Product>} becomes {@code "productMono"}<br>
+	 * {@code Flux<com.myapp.MyProduct>} becomes {@code "myProductFlux"}<br>
+	 * {@code Observable<com.myapp.MyProduct>} becomes {@code "myProductObservable"}<br>
 	 * @param method the method to generate a variable name for
 	 * @param resolvedType the resolved return type of the method
 	 * @param value the return value (may be {@code null} if not available)
 	 * @return the generated variable name
 	 */
-	public static String getVariableNameForReturnType(Method method, Class<?> resolvedType, Object value) {
+	public static String getVariableNameForReturnType(Method method, Class<?> resolvedType, @Nullable Object value) {
 		Assert.notNull(method, "Method must not be null");
 
 		if (Object.class == resolvedType) {
 			if (value == null) {
-				throw new IllegalArgumentException("Cannot generate variable name for an Object return type with null value");
+				throw new IllegalArgumentException(
+						"Cannot generate variable name for an Object return type with null value");
 			}
 			return getVariableName(value);
 		}
 
 		Class<?> valueClass;
 		boolean pluralize = false;
+		String reactiveSuffix = "";
 
 		if (resolvedType.isArray()) {
 			valueClass = resolvedType.getComponentType();
 			pluralize = true;
 		}
 		else if (Collection.class.isAssignableFrom(resolvedType)) {
-			valueClass = GenericCollectionTypeResolver.getCollectionReturnType(method);
+			valueClass = ResolvableType.forMethodReturnType(method).asCollection().resolveGeneric();
 			if (valueClass == null) {
 				if (!(value instanceof Collection)) {
-					throw new IllegalArgumentException(
-							"Cannot generate variable name for non-typed Collection return type and a non-Collection value");
+					throw new IllegalArgumentException("Cannot generate variable name " +
+							"for non-typed Collection return type and a non-Collection value");
 				}
 				Collection<?> collection = (Collection<?>) value;
 				if (collection.isEmpty()) {
-					throw new IllegalArgumentException(
-							"Cannot generate variable name for non-typed Collection return type and an empty Collection value");
+					throw new IllegalArgumentException("Cannot generate variable name " +
+							"for non-typed Collection return type and an empty Collection value");
 				}
 				Object valueToCheck = peekAhead(collection);
 				valueClass = getClassForValue(valueToCheck);
@@ -198,27 +204,32 @@ public abstract class Conventions {
 		}
 		else {
 			valueClass = resolvedType;
+			ReactiveAdapter adapter = ReactiveAdapterRegistry.getSharedInstance().getAdapter(valueClass);
+			if (adapter != null && !adapter.getDescriptor().isNoValue()) {
+				reactiveSuffix = ClassUtils.getShortName(valueClass);
+				valueClass = ResolvableType.forMethodReturnType(method).getGeneric().toClass();
+			}
 		}
 
 		String name = ClassUtils.getShortNameAsProperty(valueClass);
-		return (pluralize ? pluralize(name) : name);
+		return (pluralize ? pluralize(name) : name + reactiveSuffix);
 	}
 
 	/**
-	 * Convert {@code String}s in attribute name format (lowercase, hyphens separating words)
-	 * into property name format (camel-cased). For example, {@code transaction-manager} is
-	 * converted into {@code transactionManager}.
+	 * Convert {@code String}s in attribute name format (e.g. lowercase, hyphens
+	 * separating words) into property name format (camel-case). For example
+	 * {@code transaction-manager} becomes {@code "transactionManager"}.
 	 */
 	public static String attributeNameToPropertyName(String attributeName) {
 		Assert.notNull(attributeName, "'attributeName' must not be null");
 		if (!attributeName.contains("-")) {
 			return attributeName;
 		}
-		char[] chars = attributeName.toCharArray();
-		char[] result = new char[chars.length -1]; // not completely accurate but good guess
+		char[] result = new char[attributeName.length() -1]; // not completely accurate but good guess
 		int currPos = 0;
 		boolean upperCaseNext = false;
-		for (char c : chars) {
+		for (int i = 0; i < attributeName.length(); i++ ) {
+			char c = attributeName.charAt(i);
 			if (c == '-') {
 				upperCaseNext = true;
 			}
@@ -234,9 +245,9 @@ public abstract class Conventions {
 	}
 
 	/**
-	 * Return an attribute name qualified by the supplied enclosing {@link Class}. For example,
-	 * the attribute name '{@code foo}' qualified by {@link Class} '{@code com.myapp.SomeClass}'
-	 * would be '{@code com.myapp.SomeClass.foo}'
+	 * Return an attribute name qualified by the given enclosing {@link Class}.
+	 * For example the attribute name '{@code foo}' qualified by {@link Class}
+	 * '{@code com.myapp.SomeClass}' would be '{@code com.myapp.SomeClass.foo}'
 	 */
 	public static String getQualifiedAttributeName(Class<?> enclosingClass, String attributeName) {
 		Assert.notNull(enclosingClass, "'enclosingClass' must not be null");
@@ -246,11 +257,10 @@ public abstract class Conventions {
 
 
 	/**
-	 * Determines the class to use for naming a variable that contains
-	 * the given value.
-	 * <p>Will return the class of the given value, except when
-	 * encountering a JDK proxy, in which case it will determine
-	 * the 'primary' interface implemented by that proxy.
+	 * Determine the class to use for naming a variable containing the given value.
+	 * <p>Will return the class of the given value, except when encountering a
+	 * JDK proxy, in which case it will determine the 'primary' interface
+	 * implemented by that proxy.
 	 * @param value the value to check
 	 * @return the class to use for naming a variable
 	 */
@@ -259,7 +269,7 @@ public abstract class Conventions {
 		if (Proxy.isProxyClass(valueClass)) {
 			Class<?>[] ifcs = valueClass.getInterfaces();
 			for (Class<?> ifc : ifcs) {
-				if (!IGNORED_INTERFACES.contains(ifc)) {
+				if (!ClassUtils.isJavaLanguageInterface(ifc)) {
 					return ifc;
 				}
 			}
@@ -280,8 +290,8 @@ public abstract class Conventions {
 	}
 
 	/**
-	 * Retrieves the {@code Class} of an element in the {@code Collection}.
-	 * The exact element for which the {@code Class} is retreived will depend
+	 * Retrieve the {@code Class} of an element in the {@code Collection}.
+	 * The exact element for which the {@code Class} is retrieved will depend
 	 * on the concrete {@code Collection} implementation.
 	 */
 	private static <E> E peekAhead(Collection<E> collection) {
