@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2019 the original author or authors.
+ * Copyright 2002-2021 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -40,8 +40,8 @@ public class InlineMap extends SpelNodeImpl {
 	private TypedValue constant;
 
 
-	public InlineMap(int pos, SpelNodeImpl... args) {
-		super(pos, args);
+	public InlineMap(int startPos, int endPos, SpelNodeImpl... args) {
+		super(startPos, endPos, args);
 		checkIfConstant();
 	}
 
@@ -140,13 +140,13 @@ public class InlineMap extends SpelNodeImpl {
 		int count = getChildCount();
 		for (int c = 0; c < count; c++) {
 			if (c > 0) {
-				sb.append(",");
+				sb.append(',');
 			}
 			sb.append(getChild(c++).toStringAST());
-			sb.append(":");
+			sb.append(':');
 			sb.append(getChild(c).toStringAST());
 		}
-		sb.append("}");
+		sb.append('}');
 		return sb.toString();
 	}
 
