@@ -1,11 +1,11 @@
 /*
- * Copyright 2002-2016 the original author or authors.
+ * Copyright 2002-2021 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -20,7 +20,7 @@ import java.time.format.DateTimeFormatter;
 import java.util.Locale;
 
 import org.springframework.core.NamedThreadLocal;
-import org.springframework.lang.UsesJava8;
+import org.springframework.lang.Nullable;
 
 /**
  * A holder for a thread-local user {@link DateTimeContext}.
@@ -29,11 +29,14 @@ import org.springframework.lang.UsesJava8;
  * @since 4.0
  * @see org.springframework.context.i18n.LocaleContextHolder
  */
-@UsesJava8
 public final class DateTimeContextHolder {
 
 	private static final ThreadLocal<DateTimeContext> dateTimeContextHolder =
-			new NamedThreadLocal<DateTimeContext>("DateTimeContext");
+			new NamedThreadLocal<>("DateTimeContext");
+
+
+	private DateTimeContextHolder() {
+	}
 
 
 	/**
@@ -48,7 +51,7 @@ public final class DateTimeContextHolder {
 	 * @param dateTimeContext the current DateTimeContext,
 	 * or {@code null} to reset the thread-bound context
 	 */
-	public static void setDateTimeContext(DateTimeContext dateTimeContext) {
+	public static void setDateTimeContext(@Nullable DateTimeContext dateTimeContext) {
 		if (dateTimeContext == null) {
 			resetDateTimeContext();
 		}
@@ -61,19 +64,19 @@ public final class DateTimeContextHolder {
 	 * Return the DateTimeContext associated with the current thread, if any.
 	 * @return the current DateTimeContext, or {@code null} if none
 	 */
+	@Nullable
 	public static DateTimeContext getDateTimeContext() {
 		return dateTimeContextHolder.get();
 	}
 
-
 	/**
-	 * Obtain a DateTimeFormatter with user-specific settings applied to the given base Formatter.
+	 * Obtain a DateTimeFormatter with user-specific settings applied to the given base formatter.
 	 * @param formatter the base formatter that establishes default formatting rules
 	 * (generally user independent)
 	 * @param locale the current user locale (may be {@code null} if not known)
 	 * @return the user-specific DateTimeFormatter
 	 */
-	public static DateTimeFormatter getFormatter(DateTimeFormatter formatter, Locale locale) {
+	public static DateTimeFormatter getFormatter(DateTimeFormatter formatter, @Nullable Locale locale) {
 		DateTimeFormatter formatterToUse = (locale != null ? formatter.withLocale(locale) : formatter);
 		DateTimeContext context = getDateTimeContext();
 		return (context != null ? context.getFormatter(formatterToUse) : formatterToUse);
