@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2018 the original author or authors.
+ * Copyright 2002-2022 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -88,10 +88,10 @@ public class NotificationListenerRegistrar extends NotificationListenerHolder
 	}
 
 	/**
-	 * Allow Map access to the environment to be set for the connector,
+	 * Allow {@code Map} access to the environment to be set for the connector,
 	 * with the option to add or override specific entries.
 	 * <p>Useful for specifying entries directly, for example via
-	 * "environment[myKey]". This is particularly useful for
+	 * {@code environment[myKey]}. This is particularly useful for
 	 * adding or overriding entries in child bean definitions.
 	 */
 	@Nullable
@@ -143,7 +143,7 @@ public class NotificationListenerRegistrar extends NotificationListenerHolder
 			this.actualObjectNames = getResolvedObjectNames();
 			if (this.actualObjectNames != null) {
 				if (logger.isDebugEnabled()) {
-					logger.debug("Registering NotificationListener for MBeans " + Arrays.asList(this.actualObjectNames));
+					logger.debug("Registering NotificationListener for MBeans " + Arrays.toString(this.actualObjectNames));
 				}
 				for (ObjectName actualObjectName : this.actualObjectNames) {
 					this.server.addNotificationListener(
